@@ -1,11 +1,10 @@
-/* $Id: board.cc,v 1.121 2008/02/08 14:25:50 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -72,12 +71,6 @@ Board::Board() : WidgetProxy() {
   FlipInvert=false;
   EditMode=false;
   UpdateClockAfterConfigure = false;
-  jx=jy=-1;
-  jpxv = 0; jpyv = 0;
-
-  jsx=jsy=-1;
-  jsvx=jsvy=0;
-  jstid=-1;
 
   AnimationTimeout = -1;
 
@@ -331,8 +324,10 @@ void Board::updateClock() {
 }
 
 void RootBoard::clockString(int val,char *z,int maxlen) {
-  static char *minus = "-";
-  int cv,mr,neg,H,M,S;
+  static const char *minus = "-";
+  const int mrdiv[4] = { 1, 100, 10, 1 };
+  char fmt[32];
+  int cv,mr,neg,H,M,S,TS, ed;
   cv=val;
   if (cv<0) {
     cv=-cv;
@@ -347,16 +342,28 @@ void RootBoard::clockString(int val,char *z,int maxlen) {
   M=(cv-3600*H)/60;
   S=(cv-3600*H)%60;
 
-  if (H) {
-    snprintf(z,maxlen,"%s%d:%.2d:%.2d",
-	     neg?minus:&minus[1],H,M,S);
-  } else {
-    if (mr && M==0)
-      snprintf(z,maxlen,"%s%.2d:%.2d.%.3d",
-	       neg?minus:&minus[1],M,S,mr);
-    else
+  TS = H*3600 + M*60 + S;
+  ed = global.MsecDigits;
+  if (ed > 3) ed = 3; if (ed<1) mr=0;
+  if (ed < 0) ed = 0;
+  mr /= mrdiv[ed];
+
+  if (TS >= global.MsecThreshold || mr==0) {
+    if (H!=0) {
+      snprintf(z,maxlen,"%s%d:%.2d:%.2d",
+	       neg?minus:&minus[1],H,M,S);
+    } else {
       snprintf(z,maxlen,"%s%.2d:%.2d",
 	       neg?minus:&minus[1],M,S);
+    }
+  } else {
+    if (H==0) {
+      snprintf(fmt,32,"%%s%%.2d:%%.2d.%%.%dd", ed);
+      snprintf(z,maxlen,fmt,neg?minus:&minus[1],M,S,mr);
+    } else {
+      snprintf(fmt,32,"%%s%%d:%%.2d:%%.2d.%%.%dd", ed);
+      snprintf(z,maxlen,fmt,neg?minus:&minus[1],H,M,S,mr);
+    }
   }
 }
 
@@ -656,7 +663,7 @@ void Board::queueRepaint() {
     repaint();
 }
 
-void Board::setInfo(int idx,char *s) {
+void Board::setInfo(int idx,const char *s) {
   global.debug("Board","setInfo",s);
   info[idx%6]=s;
   clock_ch=1;
@@ -702,7 +709,7 @@ void Board::update(bool sndflag) {
 	  sok=true; // sound ok
 	  fake=position;
 	  fake.intersection(rpv);
-	  rate = global.SmootherAnimation ? 5 : 3;
+	  rate = global.SmootherAnimation ? 8 : 4;
 	  for(i=0;i<LastMove.size();i++) {
 	    if (ef)
 	      ap=new FlatAnimatedPiece(LastMove[i]->Piece,
@@ -728,12 +735,10 @@ void Board::update(bool sndflag) {
 	    gotAnimationLoop=1;
 	    currently_rendered.invalidate();
 	    board_configure_event(yidget,0,(gpointer)this);
-	    if (rate==3)
-	      tag=gtk_timeout_add(global.UseVectorPieces?40:70,
-				  board_animate,this);
+	    if (rate==4)
+	      tag=gtk_timeout_add(30,board_animate,this);
 	    else
-	      tag=gtk_timeout_add(global.UseVectorPieces?30:40,
-				  board_animate,this);
+	      tag=gtk_timeout_add(15,board_animate,this);
 	    AnimationTimeout = tag;
 	  }
 	}
@@ -878,31 +883,6 @@ void Board::walkForwardAll() {
   thaw();
 }
 
-void Board::drawJoystickCursor(GdkGC *gc) {
-  double arrow[7][2] =
-    { { 0, 0}, {0,90}, {18,68}, {40,108}, 
-      { 52,100 }, {30, 63}, {63, 63} };
-  GdkPoint myarrow[7];
-  int i;
-
-  if (jsx >= 0) {
-
-    for(i=0;i<7;i++) {
-      myarrow[i].x = (gint) (4 + jsx + arrow[i][0] * sqside / 128.0);
-      myarrow[i].y = (gint) (4 + jsy + arrow[i][1] * sqside / 128.0);
-    }
-
-    gdk_rgb_gc_set_foreground(gc,0x000000);
-    gdk_draw_polygon(yidget->window,gc,TRUE,myarrow,7);
-    for(i=0;i<7;i++) { myarrow[i].x -= 4; myarrow[i].y -= 4; }
-    gdk_rgb_gc_set_foreground(gc,0x00ff00);
-    gdk_draw_polygon(yidget->window,gc,TRUE,myarrow,7);
-    gdk_rgb_gc_set_foreground(gc,0x000000);
-    gdk_draw_polygon(yidget->window,gc,FALSE,myarrow,7);
-
-  }
-}
-
 void Board::outlineRectangle(GdkGC *gc, int x,int y,int color,int pen) {
   int i,j,k,X,Y,S;
   gdk_rgb_gc_set_foreground(gc,color);
@@ -1106,131 +1086,6 @@ void Board::popupProtocolMenu(int x,int y, guint32 etime) {
 #endif
 }
 
-gboolean board_joycursor(gpointer data) {
-  Board *me;
-  static double accel[5] = { 0.2, 0.4, 0.8, 1.2, 1.5 };
-  static int acc = 0;
-  static Timestamp prev;
-  Timestamp cur;
-  double elapsed;
-
-  me = (Board *) data;
-  cur = Timestamp::now();
-
-  if (me->jsx < 0) {
-    me->jsx = me->sqw / 2;
-    me->jsy = me->sqh / 2;
-    acc = 0;
-  } else {
-    elapsed = cur-prev;
-    if (elapsed < 0.50 && me->jsnt==0) {
-      me->jsx += (int) (accel[acc] * global.JSSpeed * 5.0 * (me->jsvx / 32768.0) * (elapsed/0.060));
-      me->jsy += (int) (accel[acc] * global.JSSpeed * 5.0 * (me->jsvy / 32768.0) * (elapsed/0.060));
-      if (me->jsx < 0)       me->jsx = 0;
-      if (me->jsx > me->sqw) me->jsx = me->sqw;
-      if (me->jsy < 0)       me->jsy = 0;
-      if (me->jsy > me->sqh) me->jsy = me->sqh;
-      ++acc;
-      if (acc>4) acc=4;
-      me->queueRepaint();
-    }
-  }
-
-  if (me->jsnt!=0) acc=0; 
-  me->jsnt = 0;
-  prev = cur;
-  if (global.JSMode!=1) {
-    me->jstid = -1;
-    return FALSE;
-  } else return TRUE;
-}
-
-void Board::joystickCursor(int axis, int value) {
-
-  // continuous cursor
-  if (global.JSMode == 1) {
-    if (axis==0) jsvx = value;
-    if (axis==1) jsvy = value;
-    if (jstid<0 && (jsvx!=0 || jsvy!=0)) {
-      jsnt = 1;
-      jstid = gtk_timeout_add(60,board_joycursor,(gpointer) this);
-    } else if (jstid>=0 && (jsvx==0 && jsvy==0)) {
-      gtk_timeout_remove(jstid);
-      jstid = -1;
-    }
-  }
-
-  if (effectiveFlip()) value = -value;   
-  if (value < 0) value = -1;
-  if (value > 0) value = 1;
-  
-
-  if (jx < 0) {
-    jx = 3; jy = 3; queueRepaint();
-  } else {
-
-    switch(axis) {
-    case 0: // X
-      if (jpxv != value) {
-	if (value < 0) jx--; if (jx<0) jx=0;
-	if (value > 0) jx++; if (jx>7) jx=7;
-	jpxv = value;
-      }
-      break;
-    case 1: // Y
-      if (jpyv != value) {
-	if (value < 0) jy++; if (jy>7) jy=7;
-	if (value > 0) jy--; if (jy<0) jy=0;
-	jpyv = value;
-      }
-      break;
-    }
-    queueRepaint();
-  }
-
-}
-
-void Board::joystickSelect() {
-
-  int x,y;
-
-  if (global.JSMode == 0) {
-    x = jx;
-    y = jy;
-    
-    if (sp > 1) sp=0;
-    if ( (sp == 1) && (x==ax[0]) && (y==ay[0]) )
-      sp=0;
-    else {
-      
-      if ((sp == 1) &&
-	  ( (position.getPiece(ax[0],ay[0])&COLOR_MASK) ==
-	    (position.getPiece(x,y)&COLOR_MASK)) &&
-	  (position.getPiece(x,y)!=EMPTY) && (allowMove) )
-	sp=0;
-      
-      ax[sp]=x; ay[sp]=y;
-      ++sp;
-    }
-    
-    if ((sp==2)&&(mygame)) {
-      sendMove();
-    } else {
-      premoveq=0;
-      hilite_ch=1;
-      repaint();
-    }
-  } else {
-    GdkEventButton be;
-    be.x = jsx;
-    be.y = jsy;
-    be.button = 1;
-    be.state = GDK_MOD5_MASK;
-    board_button_press_event(widget,&be,this);
-    board_button_release_event(widget,&be,this);
-  }
-}
-
 void Board::dump() {
   cerr.setf(ios::hex,ios::basefield);
   cerr.setf(ios::showbase);
@@ -1465,7 +1320,7 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
   // draw highlighted squares
 
   if (!me->gotAnimationLoop)
-    if ((me->sp)||(!me->LastMove.empty())||(me->premoveq)||(me->jx>=0)) {
+    if ((me->sp)||(!me->LastMove.empty())||(me->premoveq)) {
     
       if (!me->EditMode)
 	if ((global.HilightLastMove)&&(!me->LastMove.empty()))
@@ -1488,12 +1343,6 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
 	  me->drawBall(gc,me->pmx[0],me->pmy[0],0xff0000,5);
 	}
       }
-
-      if (global.JSMode==0 && me->jx >= 0)
-	me->outlineRectangle(gc,me->jx,me->jy,0x00ff00,8);
-
-      if (global.JSMode==1)
-	me->drawJoystickCursor(gc);
     }
   
   /* if dragging... */
@@ -2060,15 +1909,6 @@ gboolean board_button_press_event(GtkWidget *widget,GdkEventButton *be,
 
   if (be->button == 1) {
 
-    if (be->state != GDK_MOD5_MASK) {
-      me->jx = -1;
-      me->jy = -1;
-      me->jpxv = 0;
-      me->jpyv = 0;
-      me->jsx = -1;
-      me->jsy = -1;
-    }
-
     me->dr_active=true;
     me->dr_fto=true;
     me->dr_fromstock=false;
