@@ -1,11 +1,10 @@
-/* $Id: clock.h,v 1.15 2008/02/08 14:25:50 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -113,7 +112,7 @@ class TimeControl {
   void fromSerialization(const char *s);
 
   static void secondsToString(char *dest, int maxlen, int secs, bool units=false);
-  static int  stringToSeconds(char *src);
+  static int  stringToSeconds(const char *src);
 
   void setSecondsPerMove(int seconds);
   void setIcs(int base /* secs */, int increment /* secs */);
