@@ -1,11 +1,10 @@
-/* $Id: dlg_prefs.cc,v 1.36 2003/11/22 04:55:28 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,16 +30,17 @@
 #include "dlg_prefs.h"
 #include "global.h"
 #include "tstring.h"
+#include "clock.h"
 
-#include "snd_test.xpm"
-#include "snd_edit.xpm"
+#include "xpm/snd_test.xpm"
+#include "xpm/snd_edit.xpm"
 
 // 3700 widgets for the price of 3699. Only today at dlg_prefs.cc, the
 // Widget Warehouse.
 
 typedef void (*sigfunc)(int);
 
-char * PreferencesDialog::FontSample[NFONTS] = {
+const char * PreferencesDialog::FontSample[NFONTS] = {
   "52:38",
   "White Capablanca",
   "38. N@c5+",  
@@ -51,6 +51,9 @@ char * PreferencesDialog::FontSample[NFONTS] = {
 PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   GtkWidget *v,*hs,*ok,*apply,*cancel,*bhb,*nb;
 
+  // scroll containers for tabs
+  GtkWidget *scr[6]; // ui, ics, colors, fonts, sounds, autosave
+
   // gui
   GtkWidget *uil,*uiv,*uih,*tfr,*tr[4],*tv,*uih3,*uih4,*uih5;
   GSList *tl;
@@ -59,19 +62,21 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   GtkWidget *fl, *fv, *ft, *fn[NFONTS], *fb[NFONTS], *frv;
 
   // -- ics
-  GtkWidget *icv, *icl, *ict1, *icl1, *ifr1, *ifr2, *ifv1, *ifv2, *icl2, *icl3;
+  GtkWidget *icv, *icl, *ict1, *icl1, *ifr1, *ifr2, *ifv1, *ifv2, 
+    *icl2, *icl3, *icl4, *icl5, *icl6, *spf, *sph;
+  GSList *spl;
 
   // -- sound
-  static char *zcaption[]={N_("Opponent Moved"),
-			   N_("Draw Offered"),
-			   N_("Private Tell"),
-			   N_("ICS Challenge"),
-			   N_("Time Running Out"),
-                           N_("Game Won"), 
-			   N_("Game Lost"), 
-			   N_("Game Started"),
-                           N_("Obs'vd Game Ended"), 
-			   N_("Move made (Obs'vd/Exm'd Games)") };
+  static const char *zcaption[]={N_("Opponent Moved"),
+				 N_("Draw Offered"),
+				 N_("Private Tell"),
+				 N_("ICS Challenge"),
+				 N_("Time Running Out"),
+				 N_("Game Won"), 
+				 N_("Game Lost"), 
+				 N_("Game Started"),
+				 N_("Obs'vd Game Ended"), 
+				 N_("Move made (Obs'vd/Exm'd Games)") };
 
   GtkWidget *sv,*sl,*zf,*zv,*zt,*zl,*zhe[N_SOUND_EVENTS], *zbb[N_SOUND_EVENTS];
   GtkStyle *style;
@@ -90,7 +95,7 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   for(i=0;i< N_SOUND_EVENTS;i++)
     sndcopy[i]=global.sndevents[i];
 
-  gtk_window_set_default_size(GTK_WINDOW(widget),450,300);
+  gtk_window_set_default_size(GTK_WINDOW(widget),600,550);
 
   v=gtk_vbox_new(FALSE,0);
   gtk_container_add(GTK_CONTAINER(widget),v);
@@ -108,7 +113,7 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   tfr=gtk_frame_new(_("Tab Position"));
   gtk_box_pack_start(GTK_BOX(uiv),tfr,FALSE,FALSE,4);
   gtk_frame_set_shadow_type(GTK_FRAME(tfr),GTK_SHADOW_ETCHED_IN);
-  tv=gtk_vbox_new(FALSE,2);
+  tv=gtk_hbox_new(FALSE,2);
 
   tr[0]=gtk_radio_button_new_with_label(0,_("Right"));
   tl=gtk_radio_button_group(GTK_RADIO_BUTTON(tr[0]));
@@ -136,7 +141,7 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   gtk_box_pack_start(GTK_BOX(uih),dsq->widget,FALSE,FALSE,4); 
 
   gtk_box_pack_start(GTK_BOX(uiv),uih,FALSE,FALSE,4);
-  Gtk::show(plainb,uih,0);
+  Gtk::show(plainb,uih,NULL);
 
   gtset(GTK_TOGGLE_BUTTON(plainb), global.PlainSquares?1:0);
 
@@ -157,15 +162,16 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   gtk_box_pack_start(GTK_BOX(uiv),uih3,FALSE,FALSE,4);
   gtk_box_pack_start(GTK_BOX(uiv),uih4,FALSE,FALSE,4);
 
-  Gtk::show(smoothb, dhsb, aqbar, uih3, uih4, uih5, 0);
+  Gtk::show(smoothb, dhsb, aqbar, uih3, uih4, uih5, NULL);
 
   gtset(GTK_TOGGLE_BUTTON(smoothb), global.SmootherAnimation?1:0);
   gtset(GTK_TOGGLE_BUTTON(dhsb), global.DrawHouseStock?1:0);
   gtset(GTK_TOGGLE_BUTTON(aqbar), global.ShowQuickbar?1:0);
 
-  Gtk::show(tv, tfr, uiv, uil, 0);
+  scr[0] = scrollBox(uiv);
+  Gtk::show(tv, tfr, uiv, uil, NULL);
 
-  gtk_notebook_append_page(GTK_NOTEBOOK(nb),uiv,uil);
+  gtk_notebook_append_page(GTK_NOTEBOOK(nb),scr[0],uil);
   gtset(GTK_TOGGLE_BUTTON(tr[global.TabPos]), TRUE);
 
   // ics ===============================================================================
@@ -174,10 +180,13 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
 
   gtk_container_set_border_width(GTK_CONTAINER(icv),6);
 
-  ifr1=gtk_frame_new(_("Seek Table"));
+  ifr1=gtk_frame_new(_("Seek Graph"));
   gtk_frame_set_shadow_type(GTK_FRAME(ifr1),GTK_SHADOW_ETCHED_IN);
   ifr2=gtk_frame_new(_("Channel Tells"));
   gtk_frame_set_shadow_type(GTK_FRAME(ifr2),GTK_SHADOW_ETCHED_IN);
+  spf=gtk_frame_new(_("Non-ASCII Character Filtering"));
+  gtk_frame_set_shadow_type(GTK_FRAME(spf),GTK_SHADOW_ETCHED_IN);
+  sph=gtk_hbox_new(FALSE,2);
 
   ifv1=gtk_vbox_new(FALSE,2);
   ifv2=gtk_vbox_new(FALSE,2);
@@ -190,6 +199,9 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   autologb=gtk_check_button_new_with_label(_("Run autofics.pl script after connecting to FICS"));
   gtk_box_pack_start(GTK_BOX(icv),autologb,FALSE,FALSE,2);
 
+  showts=gtk_check_button_new_with_label(_("Show timestamps for text lines"));
+  gtk_box_pack_start(GTK_BOX(icv),showts,FALSE,FALSE,2);
+
   /*
   allob[0]=gtk_check_button_new_with_label(FIXTRAN("Show observers on played games"));
   allob[1]=gtk_check_button_new_with_label(FIXTRAN("Show observers on observed games"));
@@ -198,12 +210,28 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   gtk_box_pack_start(GTK_BOX(icv),allob[1],FALSE,FALSE,2);
   */
 
+  gtk_box_pack_start(GTK_BOX(icv),spf,FALSE,FALSE,2);
+  special[0] = gtk_radio_button_new_with_label(0,_("None"));
+  spl=gtk_radio_button_group(GTK_RADIO_BUTTON(special[0]));
+  special[1] = gtk_radio_button_new_with_label(spl,_("Truncate"));
+  spl=gtk_radio_button_group(GTK_RADIO_BUTTON(special[1]));
+  special[2] = gtk_radio_button_new_with_label(spl,_("Use underscores"));
+  spl=gtk_radio_button_group(GTK_RADIO_BUTTON(special[2]));
+  special[3] = gtk_radio_button_new_with_label(spl,_("Soft translate"));
+  spl=gtk_radio_button_group(GTK_RADIO_BUTTON(special[3]));
+
+  gtk_container_add(GTK_CONTAINER(spf),sph);
+  for(i=0;i<4;i++) {
+    gtk_box_pack_start(GTK_BOX(sph),special[i],FALSE,FALSE,2);
+    gshow(special[i]);
+  }
+
   gtk_box_pack_start(GTK_BOX(icv),ifr1,FALSE,FALSE,2);
 
-  sgb=gtk_check_button_new_with_label(_("Dynamic Seek Table"));
+  sgb=gtk_check_button_new_with_label(_("Dynamic Seek Graph"));
   gtk_box_pack_start(GTK_BOX(ifv1),sgb,FALSE,FALSE,2);
 
-  hsb=gtk_check_button_new_with_label(_("Inhibit seek lines on console when Seek Table is active"));
+  hsb=gtk_check_button_new_with_label(_("Inhibit seek lines on console when Seek Graph is active"));
   gtk_box_pack_start(GTK_BOX(ifv1),hsb,FALSE,FALSE,2);
 
   gtk_box_pack_start(GTK_BOX(icv),ifr2,FALSE,FALSE,2);
@@ -217,11 +245,11 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   wget=gtk_check_button_new_with_label(_("Retrieve ICS Channel Lists from eboard.sf.net"));
   gtk_box_pack_start(GTK_BOX(ifv2),wget,FALSE,FALSE,0);
 
-  ict1=gtk_table_new(2,3,FALSE);
+  ict1=gtk_table_new(2,5,FALSE);
 
   icl1=gtk_label_new(_("Scrollback limit (0 = unlimited) :"));
   sbacke=gtk_entry_new();
-  sprintf(z,"%d",global.ScrollBack);
+  snprintf(z,64,"%d",global.ScrollBack);
   gtk_entry_set_text(GTK_ENTRY(sbacke),z);
 
   gtk_table_attach_defaults(GTK_TABLE(ict1), icl1, 0,1, 0,1);
@@ -229,24 +257,53 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
 
   icl2=gtk_label_new(_("Warn when own clock is below: "));
   lowtime=gtk_entry_new();
-  sprintf(z,"%d",global.LowTimeWarningLimit);
+  snprintf(z,64,"%d",global.LowTimeWarningLimit);
   gtk_entry_set_text(GTK_ENTRY(lowtime),z);
   icl3=gtk_label_new(_("seconds."));
 
+  icl4=gtk_label_new(_("Show milliseconds when below: "));
+  icl6=gtk_label_new(_("(HH:MM:SS)"));
+  msecth=gtk_entry_new();
+  {
+    int hh,mm,ss;
+    hh = global.MsecThreshold / 3600;
+    mm = (global.MsecThreshold % 3600) / 60;
+    ss = (global.MsecThreshold % 3600) % 60;
+    snprintf(z,64,"%.2d:%.2d:%.2d",hh,mm,ss);
+    gtk_entry_set_text(GTK_ENTRY(msecth),z);
+  }
+
+  icl5=gtk_label_new(_("Subsecond precision digits: "));
+  msecdig = gtk_entry_new();
+  snprintf(z,64,"%d",global.MsecDigits);
+  gtk_entry_set_text(GTK_ENTRY(msecdig),z);
+
   gtk_table_attach_defaults(GTK_TABLE(ict1), icl2, 0,1, 1,2);
   gtk_table_attach_defaults(GTK_TABLE(ict1), lowtime, 1,2, 1,2);
   gtk_table_attach_defaults(GTK_TABLE(ict1), icl3, 2,3, 1,2);
 
+  gtk_table_attach_defaults(GTK_TABLE(ict1), icl4, 0,1, 2,3);
+  gtk_table_attach_defaults(GTK_TABLE(ict1), msecth, 1,2, 2,3);
+  gtk_table_attach_defaults(GTK_TABLE(ict1), icl6, 2,3, 2,3);
+
+  gtk_table_attach_defaults(GTK_TABLE(ict1), icl5, 0,1, 3,4);
+  gtk_table_attach_defaults(GTK_TABLE(ict1), msecdig, 1,2, 3,4);
+
   gtk_box_pack_start(GTK_BOX(icv),ict1,FALSE,FALSE,2);
 
+  scr[1] = scrollBox(icv);
   Gtk::show(icl1, sbacke, icl2, lowtime, icl3, ict1,
 	    ifr1, ifr2, ifv1, ifv2,
-	    icl, showratb, autologb, /*allob[0], allob[1],*/
-	    sgb, chsb, coct, wget, hsb, icv, 0);
-
-  gtk_notebook_append_page(GTK_NOTEBOOK(nb),icv,icl);
+	    icl, showratb, autologb, showts, /*allob[0], allob[1],*/
+	    sgb, chsb, coct, wget, hsb, icv, spf, sph, icl4, msecth, icl5, 
+	    msecdig, icl6, NULL);
+  
+  gtk_notebook_append_page(GTK_NOTEBOOK(nb),scr[1],icl);
+  
   gtset(GTK_TOGGLE_BUTTON(showratb), global.ShowRating?1:0);
   gtset(GTK_TOGGLE_BUTTON(autologb), global.FicsAutoLogin?1:0);
+  gtset(GTK_TOGGLE_BUTTON(showts),   global.ShowTimestamp?1:0);
+  gtset(GTK_TOGGLE_BUTTON(special[global.SpecialChars]), TRUE);
   /*
   gtset(GTK_TOGGLE_BUTTON(allob[0]), global.IcsAllObPlayed?1:0);
   gtset(GTK_TOGGLE_BUTTON(allob[1]), global.IcsAllObObserved?1:0);
@@ -300,8 +357,10 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   tcdef=gtk_button_new_with_label(_("Revert to Defaults"));
   gtk_box_pack_start(GTK_BOX(tcv2),tcdef,FALSE,FALSE,4);
 
-  Gtk::show(preview->widget, tcdef, tch, tcv, tcv2, tcl, 0);
-  gtk_notebook_append_page(GTK_NOTEBOOK(nb),tch,tcl);
+  scr[2] = scrollBox(tch);
+  Gtk::show(preview->widget, tcdef, tch, tcv, tcv2, tcl, NULL);
+
+  gtk_notebook_append_page(GTK_NOTEBOOK(nb),scr[2],tcl);
 
   gtk_signal_connect(GTK_OBJECT(tcdef),"clicked",
 		     GTK_SIGNAL_FUNC(prefs_defcolor),(gpointer)this);
@@ -335,7 +394,7 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   gtk_entry_set_text(GTK_ENTRY(fn[3]),global.ConsoleFont);
   fb[3]=gtk_button_new_with_label(_("Choose..."));
 
-  fm[4]=new BoxedLabel(_("Seek Table Font"));
+  fm[4]=new BoxedLabel(_("Seek Graph Font"));
   gtk_entry_set_text(GTK_ENTRY(fn[4]),global.SeekFont);
   fb[4]=gtk_button_new_with_label(_("Choose..."));
 
@@ -357,8 +416,10 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
     efont[i]=fn[i];
     xfont[i]=fb[i];
   }
-  Gtk::show(ft,fv,fl,0);
-  gtk_notebook_append_page(GTK_NOTEBOOK(nb),fv,fl);
+
+  scr[3] = scrollBox(fv);
+  Gtk::show(ft,fv,fl,NULL);
+  gtk_notebook_append_page(GTK_NOTEBOOK(nb),scr[3],fl);
 
   gtk_signal_connect(GTK_OBJECT(frv),"clicked",
 		     GTK_SIGNAL_FUNC(prefs_frevert),(gpointer)this);
@@ -425,15 +486,17 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
 
     gtset(GTK_TOGGLE_BUTTON(sndon[i]), sndcopy[i].enabled);   
     Gtk::show(sndon[i],sndd[i],zhe[i],sndtest[i],sndedit[i],pic[0][i],
-	      pic[1][i],zbb[i],0);
+	      pic[1][i],zbb[i],NULL);
 
     gtk_signal_connect(GTK_OBJECT(sndtest[i]),"clicked",
 		       GTK_SIGNAL_FUNC(prefs_sndtest), (gpointer) this);
     gtk_signal_connect(GTK_OBJECT(sndedit[i]),"clicked",
 		       GTK_SIGNAL_FUNC(prefs_sndedit), (gpointer) this);
   }
-  Gtk::show(zt,zv,zf,sv,sl,zl,0);
-  gtk_notebook_append_page(GTK_NOTEBOOK(nb),sv,sl);
+
+  scr[4] = scrollBox(sv);
+  Gtk::show(zt,zv,zf,sv,sl,zl,NULL);
+  gtk_notebook_append_page(GTK_NOTEBOOK(nb),scr[4],sl);
 
   // autosave ========================================================================
 
@@ -457,8 +520,10 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   gtk_box_pack_start(GTK_BOX(gh),gll,FALSE,FALSE,4);
   gtk_box_pack_start(GTK_BOX(gh),afn,TRUE,TRUE,4);
 
-  Gtk::show(gv,gh,gl,gll,asp,aso,afn,0);
-  gtk_notebook_append_page(GTK_NOTEBOOK(nb),gv,gl);
+  scr[5] = scrollBox(gv);
+  
+  Gtk::show(gv,gh,gl,gll,asp,aso,afn,NULL);
+  gtk_notebook_append_page(GTK_NOTEBOOK(nb),scr[5],gl);
 
   // bottom
   hs=gtk_hseparator_new();
@@ -478,7 +543,7 @@ PreferencesDialog::PreferencesDialog() : ModalDialog(N_("Preferences")) {
   gtk_box_pack_start(GTK_BOX(bhb),cancel,TRUE,TRUE,0);
   gtk_widget_grab_default(ok);
 
-  Gtk::show(ok,apply,cancel,bhb,hs,nb,v,0);
+  Gtk::show(ok,apply,cancel,bhb,hs,nb,v,NULL);
 
   setDismiss(GTK_OBJECT(cancel),"clicked");
   gtk_signal_connect(GTK_OBJECT(ok),"clicked",
@@ -511,10 +576,10 @@ void PreferencesDialog::ApplyCheckBox(GtkWidget *cb,int *curval,int *ch1, int *c
 void PreferencesDialog::ApplyEntry(GtkWidget *entry,char *curval, int sz,
 				   int *ch1, int *ch2)
 {
-  char *nv;
+  const char *nv;
   nv=gtk_entry_get_text(GTK_ENTRY(entry));
   if (strcmp(curval,nv)) {
-    strncpy(curval,nv,sz);
+    g_strlcpy(curval,nv,sz);
     if (ch1) *ch1=1;
     if (ch2) *ch2=1;
   }
@@ -533,7 +598,7 @@ void PreferencesDialog::ApplyColorButton(ColorButton *cb,int *curval,
 void PreferencesDialog::Apply() {
   int i,nval=0,changed=0,fchg=0, schg=0, psetchg=0, tbg=0;
   GtkPositionType ntp;
-  char *p;
+  const char *p;
 
   for(i=0;i<4;i++)
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tabposb[i]))) nval=i;
@@ -551,47 +616,56 @@ void PreferencesDialog::Apply() {
     global.ebook->setTabPosition(ntp);
   }
 
-  ApplyEntry(efont[0],global.ClockFont,96,&changed,0);
-  ApplyEntry(efont[1],global.PlayerFont,96,&changed,0);
-  ApplyEntry(efont[2],global.InfoFont,96,&changed,0);
+  for(i=0,nval=0;i<4;i++)
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(special[i]))) nval=i;
+
+  if (nval!=global.SpecialChars) {
+    global.SpecialChars = nval%4;
+    changed = 1;
+  }
+
+  ApplyEntry(efont[0],global.ClockFont,96,&changed,NULL);
+  ApplyEntry(efont[1],global.PlayerFont,96,&changed,NULL);
+  ApplyEntry(efont[2],global.InfoFont,96,&changed,NULL);
   ApplyEntry(efont[3],global.ConsoleFont,96,&changed,&fchg);
   ApplyEntry(efont[4],global.SeekFont,96,&changed,&schg);
 
   ApplyCheckBox(plainb, &global.PlainSquares, &changed, &psetchg);
-  ApplyCheckBox(smoothb, &global.SmootherAnimation, &changed, 0);
-  ApplyCheckBox(dhsb, &global.DrawHouseStock, &changed, 0);
-  ApplyCheckBox(aqbar, &global.ShowQuickbar, &changed, 0);
-  ApplyCheckBox(showratb, &global.ShowRating, &changed, 0);
-  ApplyCheckBox(autologb, &global.FicsAutoLogin, &changed, 0);
+  ApplyCheckBox(smoothb, &global.SmootherAnimation, &changed, NULL);
+  ApplyCheckBox(dhsb, &global.DrawHouseStock, &changed, NULL);
+  ApplyCheckBox(aqbar, &global.ShowQuickbar, &changed, NULL);
+  ApplyCheckBox(showratb, &global.ShowRating, &changed, NULL);
+  ApplyCheckBox(autologb, &global.FicsAutoLogin, &changed, NULL);
+  ApplyCheckBox(showts, &global.ShowTimestamp, &changed, &fchg);
   /*
-  ApplyCheckBox(allob[0], &global.IcsAllObPlayed, &changed, 0);
-  ApplyCheckBox(allob[1], &global.IcsAllObObserved, &changed, 0);
+  ApplyCheckBox(allob[0], &global.IcsAllObPlayed, &changed, NULL);
+  ApplyCheckBox(allob[1], &global.IcsAllObObserved, &changed, NULL);
   */
-  ApplyCheckBox(sgb, &global.IcsSeekGraph, &changed, 0);
-  ApplyCheckBox(hsb, &global.HideSeeks, &changed, 0);
-  ApplyCheckBox(chsb, &global.SplitChannels, &changed, 0);
-  ApplyCheckBox(coct, &global.ChannelsToConsoleToo, &changed, 0);
-  ApplyCheckBox(wget, &global.RetrieveChannelNames, &changed, 0);
-  ApplyCheckBox(asp, &global.AppendPlayed, &changed, 0);
-  ApplyCheckBox(aso, &global.AppendObserved, &changed, 0);
+  ApplyCheckBox(sgb, &global.IcsSeekGraph, &changed, NULL);
+  ApplyCheckBox(hsb, &global.HideSeeks, &changed, NULL);
+  ApplyCheckBox(chsb, &global.SplitChannels, &changed, NULL);
+  ApplyCheckBox(coct, &global.ChannelsToConsoleToo, &changed, NULL);
+  ApplyCheckBox(wget, &global.RetrieveChannelNames, &changed, NULL);
+  ApplyCheckBox(asp, &global.AppendPlayed, &changed, NULL);
+  ApplyCheckBox(aso, &global.AppendObserved, &changed, NULL);
 
-  ApplyEntry(afn,global.AppendFile,128,&changed,0);
+  ApplyEntry(afn,global.AppendFile,128,&changed,NULL);
 
   if (lsq->getColor() != global.LightSqColor) {
     changed=1; psetchg=1;
     global.LightSqColor=lsq->getColor();
   }
 
-  ApplyColorButton(textcb[0],&global.Colors.TextDefault,&changed,0);
-  ApplyColorButton(textcb[1],&global.Colors.TextBright,&changed,0);
-  ApplyColorButton(textcb[2],&global.Colors.PrivateTell,&changed,0);
-  ApplyColorButton(textcb[3],&global.Colors.NewsNotify,&changed,0);
-  ApplyColorButton(textcb[4],&global.Colors.Mamer,&changed,0);
-  ApplyColorButton(textcb[5],&global.Colors.KibitzWhisper,&changed,0);
-  ApplyColorButton(textcb[6],&global.Colors.Shouts,&changed,0);
-  ApplyColorButton(textcb[7],&global.Colors.Seeks,&changed,0);
-  ApplyColorButton(textcb[8],&global.Colors.ChannelTell,&changed,0);
-  ApplyColorButton(textcb[9],&global.Colors.Engine,&changed,0);
+  ApplyColorButton(textcb[0],&global.Colors.TextDefault,&changed,NULL);
+  ApplyColorButton(textcb[1],&global.Colors.TextBright,&changed,NULL);
+  ApplyColorButton(textcb[2],&global.Colors.PrivateTell,&changed,NULL);
+  ApplyColorButton(textcb[3],&global.Colors.NewsNotify,&changed,NULL);
+  ApplyColorButton(textcb[4],&global.Colors.Mamer,&changed,NULL);
+  ApplyColorButton(textcb[5],&global.Colors.KibitzWhisper,&changed,NULL);
+  ApplyColorButton(textcb[6],&global.Colors.Shouts,&changed,NULL);
+  ApplyColorButton(textcb[7],&global.Colors.Seeks,&changed,NULL);
+  ApplyColorButton(textcb[8],&global.Colors.ChannelTell,&changed,NULL);
+  ApplyColorButton(textcb[9],&global.Colors.Engine,&changed,NULL);
   ApplyColorButton(textcb[10],&global.Colors.Background,&changed,&tbg);
 
   if (dsq->getColor() != global.DarkSqColor) {
@@ -614,6 +688,20 @@ void PreferencesDialog::Apply() {
     global.LowTimeWarningLimit=nval;
   }
 
+  p=gtk_entry_get_text(GTK_ENTRY(msecth));
+  nval=TimeControl::stringToSeconds(p);
+  if (nval!=global.MsecThreshold) {
+    changed=1;
+    global.MsecThreshold=nval;
+  }
+
+  p=gtk_entry_get_text(GTK_ENTRY(msecdig));
+  nval=atoi(p);
+  if (nval!=global.MsecDigits) {
+    changed=1;
+    global.MsecDigits=nval;
+  }
+
   for(i=0;i<N_SOUND_EVENTS;i++) {
     sndcopy[i].enabled = false;
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sndon[i])))
@@ -635,10 +723,25 @@ void PreferencesDialog::Apply() {
 	(*global.BLi)->invalidate();
       (*global.BLi)->queueRepaint();
     }
-  if (fchg)
+  if (fchg) {
     global.output->updateFont();
-  if (schg && global.skgraph)
-    global.skgraph->updateFont();
+    global.updateFont();
+  }
+
+  if (changed && global.IcsSeekGraph!=0)
+    if (global.network!=NULL && global.protocol!=NULL)
+      if (global.network->isConnected())
+	global.protocol->refreshSeeks(true);
+  
+  if (changed && global.IcsSeekGraph==0 && global.skgraph2 != NULL) {
+    global.skgraph2->clear();
+  }
+  
+  if (schg && global.skgraph2!=NULL) {
+    global.skgraph2->updateFont();
+    global.skgraph2->repaint();
+  }
+
   if (tbg)
     global.output->setBackground(global.Colors.Background);
 
@@ -682,22 +785,28 @@ void prefs_ok(GtkWidget *w,gpointer data) {
   me->release();
 }
 
+void prefs_fcancel(GtkWidget *w,gpointer data) {
+  PreferencesDialog *me;
+  me=(PreferencesDialog *)data;
+  gtk_grab_remove(me->fontdlg);
+  gtk_widget_destroy(me->fontdlg);
+}
+
 void prefs_fok(GtkWidget *w,gpointer data) {
   PreferencesDialog *me;
   char *p;
   me=(PreferencesDialog *)data;
   p=gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(me->fontdlg)); 
-  if (p)
+  if (p!=NULL)
     gtk_entry_set_text(GTK_ENTRY(me->efont[me->FontBeingEdited]),p);
   gtk_grab_remove(me->fontdlg);
   gtk_widget_destroy(me->fontdlg);
-
 }
 
 void prefs_cfont(GtkWidget *w,gpointer data) {
   PreferencesDialog *me;
   GtkWidget *fd;
-  char cmd[256];
+  const gchar *cmd;
   int i, fbe;
 
   me=(PreferencesDialog *)data;  
@@ -709,13 +818,12 @@ void prefs_cfont(GtkWidget *w,gpointer data) {
       break;
     }
   if (me->FontBeingEdited < 0) {
-    cerr << "**** eboard internal error: you should never see this, email bergo@seul.org\n";
+    cerr << "**** eboard internal error: you should never see this, report it\n";
     return;
   }
   fbe = me->FontBeingEdited;
 
-  // this GTK font dialog sucks so much...
-  me->ResolveFont(gtk_entry_get_text(GTK_ENTRY(me->efont[fbe])),cmd);
+  cmd = gtk_entry_get_text(GTK_ENTRY(me->efont[fbe]));
 
   fd=gtk_font_selection_dialog_new(_("Choose Font"));
 
@@ -724,18 +832,14 @@ void prefs_cfont(GtkWidget *w,gpointer data) {
 
   gtk_font_selection_dialog_set_preview_text(GTK_FONT_SELECTION_DIALOG(fd),
 					     me->FontSample[fbe]);
-  gshow(fd);
-  gtk_grab_add(fd);
 
-  gtk_signal_connect (GTK_OBJECT (fd), "destroy",
-		      GTK_SIGNAL_FUNC (gtk_widget_destroyed),&fd);
   gtk_signal_connect (GTK_OBJECT (GTK_FONT_SELECTION_DIALOG (fd)->ok_button),
 		      "clicked", GTK_SIGNAL_FUNC(prefs_fok),data);
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FONT_SELECTION_DIALOG (fd)->cancel_button),
-			     "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy), 
-			     GTK_OBJECT (fd));
+  gtk_signal_connect(GTK_OBJECT (GTK_FONT_SELECTION_DIALOG (fd)->cancel_button),
+		     "clicked", GTK_SIGNAL_FUNC (prefs_fcancel), data);
   me->fontdlg=fd;
+  gshow(fd);
+  gtk_grab_add(fd);
 }
 
 void prefs_frevert(GtkWidget *w,gpointer data) {
@@ -748,33 +852,13 @@ void prefs_frevert(GtkWidget *w,gpointer data) {
   gtk_entry_set_text(GTK_ENTRY(me->efont[4]),DEFAULT_FONT_SEEK);
 }
 
-#include <X11/Xlib.h>
-
-void PreferencesDialog::ResolveFont(char *src,char *dest) {
-  Display *dpy;
-  char **ptr;
-  int r;
-
-  strcpy(dest,src);
-  dpy=XOpenDisplay(gdk_get_display());
-  if (dpy) {
-    ptr=XListFonts(dpy,src,1,&r);
-    if (ptr) {
-      if (r)
-	strcpy(dest,ptr[0]);
-      XFreeFontNames(ptr);
-    }
-    XCloseDisplay(dpy);
-  }
-}
-
 // sounds (the error codes are plain random ;-)
 
 void prefs_sndtest(GtkWidget *w,gpointer data) {
   PreferencesDialog *me = (PreferencesDialog *)data;
   for(int i=0;i<N_SOUND_EVENTS;i++)
     if (w == me->sndtest[i]) {
-      me->sndcopy[i].safePlay();
+      me->sndcopy[i].play();
       return;
     }
   cerr << "** eboard ** odd internal error 5B7X (email the author).\n"; 
