@@ -1,11 +1,10 @@
-/* $Id: dlg_prefs.h,v 1.32 2008/02/22 07:32:15 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2008 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,8 +30,7 @@
 #include "widgetproxy.h"
 
 class PreferencesDialog : public ModalDialog,
-                          public SoundEventChangeListener,
-                          public JoystickListener
+                          public SoundEventChangeListener
 {
  public:
   PreferencesDialog();
@@ -40,13 +38,11 @@ class PreferencesDialog : public ModalDialog,
 
   void SoundEventChanged();
 
-  virtual void joystickEvent(JoystickEventType jet, int number, int value);
-
  private:
   GtkWidget *tabposb[4],*efont[NFONTS],*fontdlg,*plainb,
     *showratb, *autologb, *showts, *sbacke, *special[4], /* *allob[2], */ 
     *asp, *aso, *afn, *sgb, *hsb, *chsb, *coct, *dhsb, *aqbar, *lowtime,
-    *wget, *smoothb, *xfont[NFONTS], 
+    *msecth, *msecdig, *wget, *smoothb, *xfont[NFONTS], 
     *sndon[N_SOUND_EVENTS],*sndtest[N_SOUND_EVENTS],*sndedit[N_SOUND_EVENTS],
     *sndd[N_SOUND_EVENTS], *jcl, *jctl, *jmode, *jspeed;
   BoxedLabel *fm[NFONTS];
@@ -56,15 +52,13 @@ class PreferencesDialog : public ModalDialog,
   int FontBeingEdited;
   int jsval[5], jstate;
 
-  static char *FontSample[NFONTS];
+  static const char *FontSample[NFONTS];
 
   void Apply();
   void ApplyCheckBox(GtkWidget *cb,int *curval,int *ch1, int *ch2);
   void ApplyEntry(GtkWidget *entry,char *curval,int sz, int *ch1, int *ch2);
   void ApplyColorButton(ColorButton *cb,int *curval,int *ch1, int *ch2);
 
-  void formatJoystickDescription();
-
   friend void prefs_ok(GtkWidget *w,gpointer data);
   friend void prefs_apply(GtkWidget *w,gpointer data);
 
@@ -77,8 +71,6 @@ class PreferencesDialog : public ModalDialog,
   friend void prefs_sndedit(GtkWidget *w,gpointer data);
 
   friend void prefs_defcolor(GtkWidget *w,gpointer data);
-
-  friend void prefs_joyctl(GtkWidget *w,gpointer data);
 };
 
 void prefs_ok(GtkWidget *w,gpointer data);
@@ -93,6 +85,5 @@ void prefs_sndtest(GtkWidget *w,gpointer data);
 void prefs_sndedit(GtkWidget *w,gpointer data);
 
 void prefs_defcolor(GtkWidget *w,gpointer data);
-void prefs_joyctl(GtkWidget *w,gpointer data);
 
 #endif
