@@ -1,11 +1,10 @@
-/* $Id: eboard.h,v 1.42 2003/11/14 01:59:42 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -68,7 +67,7 @@ class PieceProvider {
 class InputHandler {
  public:
   virtual void setPasswordMode(int pm)=0;
-  virtual void userInput(char *text)=0;
+  virtual void userInput(const char *text)=0;
   virtual void peekKeys(GtkWidget *who)=0;
   virtual int  keyPressed(int keyval, int state)=0;
   virtual void updatePrefix()=0;
@@ -76,8 +75,8 @@ class InputHandler {
 
 class ConnectionHandler {
  public:
-  virtual void openServer(char *host,int port,
-			  Protocol *protocol,char *helper)=0;
+  virtual void openServer(const char *host,int port,
+			  Protocol *protocol,const char *helper)=0;
   virtual void openServer(NetConnection *conn, Protocol *protocol)=0;
   virtual void openEngine(EngineProtocol *protocol, EngineBookmark *bm)=0;
 };
@@ -153,8 +152,14 @@ typedef enum {
   LOSERS,
   GIVEAWAY,
   ATOMIC,
+  WILDFR,
+  WILDCASTLE,
+  WILDNOCASTLE,
 } variant;
 
+#define IS_WILD(v)    (((v)==WILD)||((v)==WILDFR)||((v)==WILDCASTLE)||((v)==WILDNOCASTLE))
+#define IS_NOT_WILD(v) (((v)!=WILD)&&((v)!=WILDFR)&&((v)!=WILDCASTLE)&&((v)!=WILDNOCASTLE))
+
 typedef enum {
   WHITE_WIN,
   BLACK_WIN,
@@ -171,16 +176,16 @@ typedef enum {
   IM_RESET=5,
 } Importance;
 
-#define CLOCK_UNCHANGED (-1000000)
+#define CLOCK_UNCHANGED (-100000000)
 
 // number of configurable fonts
 #define NFONTS 5
 
-#define DEFAULT_FONT_CLOK "-*-lucida-bold-r-normal-*-34-*-*-*-*-*-*"
-#define DEFAULT_FONT_PLYR "-*-helvetica-bold-r-normal--18-*-*-*-*-*-*"
-#define DEFAULT_FONT_INFO "-*-helvetica-medium-r-normal--12-*-*-*-*-*-*"
-#define DEFAULT_FONT_CONS "-misc-fixed-medium-r-normal--14-*-*-*-*-*-*"
-#define DEFAULT_FONT_SEEK "-misc-fixed-medium-r-normal--14-*-*-*-*-*-*"
+#define DEFAULT_FONT_CLOK "Sans Bold 26"
+#define DEFAULT_FONT_PLYR "Sans 14"
+#define DEFAULT_FONT_INFO "Sans 10"
+#define DEFAULT_FONT_CONS "Bitstream Vera Sans Mono 10"
+#define DEFAULT_FONT_SEEK "Sans 10"
 
 
 typedef enum {
@@ -210,7 +215,7 @@ typedef enum {
 #ifdef ENABLE_NLS
   #include "langs.h"
   #define _(s) langs_translate(s)
-  #define N_(s) (s)
+#define N_(s) (s)
   #define eboard_gettext(s) langs_translate(s)
 #else
   #define _(s) (s)
