@@ -1,11 +1,10 @@
-/* $Id: global.cc,v 1.98 2008/02/22 07:32:16 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2008 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -169,6 +168,8 @@ Global::Global() {
   ShowTimestamp = 0;
   ShowRating=1;
   SpecialChars = 3;
+  MsecThreshold = 60;
+  MsecDigits = 3;
 
   UseVectorPieces=0;
   CheckLegality=0;
@@ -188,8 +189,6 @@ Global::Global() {
 
   // chess machine had this and FICS admins didn't like it
   // if you turn it on, you may be committing abuse.
-  // this is DAV's fault for not implementing a decent
-  // way of tracking game observers.
   // I'm not going to delete the code. I'm just removing the
   // user-friendly controls from the config dialog and setting
   // the defaults to safe values.
@@ -197,18 +196,6 @@ Global::Global() {
   IcsAllObPlayed   = 0;
   IcsAllObObserved = 0;
 
-  JoystickFD = -1;
-
-  JSCursorAxis = 0;
-  JSBrowseAxis = 2;
-  JSMoveButton = 0;
-  JSNextTabButton = 5;
-  JSPrevTabButton = 4;  
-  JSMode  = 1;
-  JSSpeed = 4;
-
-  joycapture = NULL;
-
   ShowQuickbar=1;
   LowTimeWarningLimit=5;
   RetrieveChannelNames=1;
@@ -240,7 +227,7 @@ Global::Global() {
 
   memset(P2PName,0,64);
   env.User.copy(P2PName,63);
-
+  
   RCKeys.push_back("HilightLastMove"); //0
   RCKeys.push_back("AnimateMoves");
   RCKeys.push_back("Premove");         //2
@@ -290,13 +277,15 @@ Global::Global() {
   RCKeys.push_back("P2PName");         // 46
   RCKeys.push_back("ShowTimestamp");
   RCKeys.push_back("SpecialChars");// 48
-  RCKeys.push_back("JSCAxis");
-  RCKeys.push_back("JSBAxis"); // 50
-  RCKeys.push_back("JSMButton");
-  RCKeys.push_back("JSNTButton"); // 52
-  RCKeys.push_back("JSPTButton");
-  RCKeys.push_back("JSMode"); // 54
-  RCKeys.push_back("JSSpeed");
+  RCKeys.push_back("JSCAxis"); //          [deprecated]
+  RCKeys.push_back("JSBAxis"); // 50       [deprecated]
+  RCKeys.push_back("JSMButton"); //        [deprecated]
+  RCKeys.push_back("JSNTButton"); // 52 // [deprecated]
+  RCKeys.push_back("JSPTButton"); //       [deprecated]
+  RCKeys.push_back("JSMode"); // 54 //     [deprecated]
+  RCKeys.push_back("JSSpeed"); //          [deprecated]
+  RCKeys.push_back("MsecThreshold"); // 56
+  RCKeys.push_back("MsecDigits");
 
   PopupHelp = false;
 }
@@ -420,11 +409,11 @@ void Global::readRC() {
   string  *p;
   char    line[512],rev[128];
 
-  static char *sep=" \n\t,:\r^";
-  static char *sep2="\n\t,:\r";
-  static char *sep3=" \n\t:\r";
-  static char *sep4=",\n\r";
-  static char *sep5="\n\t:\r";
+  static const char *sep=" \n\t,:\r^";
+  static const char *sep2="\n\t,:\r";
+  static const char *sep3=" \n\t:\r";
+  static const char *sep4=",\n\r";
+  static const char *sep5="\n\t:\r";
 
   HostBookmark *hbm;
   EngineBookmark *ebm;
@@ -456,7 +445,7 @@ void Global::readRC() {
     if (!p) continue;
     if (p->at(0)=='#') continue;
 
-    for(j=0;j<=55;j++) {
+    for(j=0;j<=57;j++) {
       if (! p->compare(RCKeys[j]) ) {
 	switch(j) {
 	case  0: HilightLastMove =t.tokenvalue(sep); break;
@@ -530,16 +519,20 @@ void Global::readRC() {
                  p->copy(P2PName,63); break;
 	case 47: ShowTimestamp       =t.tokenvalue(sep); break;
 	case 48: SpecialChars        =t.tokenvalue(sep); break;
-	case 49: JSCursorAxis = t.tokenvalue(sep); break;
-	case 50: JSBrowseAxis = t.tokenvalue(sep); break;
-	case 51: JSMoveButton = t.tokenvalue(sep); break;
-	case 52: JSNextTabButton = t.tokenvalue(sep); break;
-	case 53: JSPrevTabButton = t.tokenvalue(sep); break;
-	case 54: JSMode  = t.tokenvalue(sep); break;
-	case 55: JSSpeed = t.tokenvalue(sep); break;
+	case 49:
+	case 50:
+	case 51:
+	case 52:
+	case 53:
+	case 54:
+	case 55:
+	  // 49-55: deprecated joystick-related keys
+	  break;
+	case 56: MsecThreshold = t.tokenvalue(sep); break;
+	case 57: MsecDigits = t.tokenvalue(sep); break;
 	} // switch
       } // compare
-    } // for j 0..55
+    } // for j 0..57
   } // while getline
 
   rc.close();
@@ -551,7 +544,7 @@ void Global::writeRC() {
   
   list<HostBookmark *>::iterator bi;
   list<EngineBookmark *>::iterator ei;
-  static char *tabpos="RLTB";
+  static const char *tabpos="RLTB";
   unsigned int i;
 
   if (env.Config.empty())
@@ -633,13 +626,11 @@ void Global::writeRC() {
   rc << RCKeys[46] << div << P2PName << endl;
   rc << RCKeys[47] << div << ShowTimestamp << endl;
   rc << RCKeys[48] << div << SpecialChars << endl;
-  rc << RCKeys[49] << div << JSCursorAxis << endl;
-  rc << RCKeys[50] << div << JSBrowseAxis << endl;
-  rc << RCKeys[51] << div << JSMoveButton << endl;
-  rc << RCKeys[52] << div << JSNextTabButton << endl;
-  rc << RCKeys[53] << div << JSPrevTabButton << endl;
-  rc << RCKeys[54] << div << JSMode << endl;
-  rc << RCKeys[55] << div << JSSpeed << endl;
+
+  // 49-55: deprecated (joystick)
+  
+  rc << RCKeys[56] << div << MsecThreshold << endl;
+  rc << RCKeys[57] << div << MsecDigits << endl;
 
   rc.close();
 }
@@ -722,7 +713,7 @@ void Global::opponentMoved() {
     if (AnimateMoves)
       SoundStack.push(0);
     else
-      sndevents[0].safePlay();
+      sndevents[0].play();
   }
 }
 
@@ -735,7 +726,7 @@ void Global::clearSoundStack() {
 
 void Global::flushSound() {
   if (!SoundStack.empty()) {
-    sndevents[SoundStack.top()].safePlay();
+    sndevents[SoundStack.top()].play();
     SoundStack.pop();
   }
 }
@@ -754,14 +745,14 @@ void Global::moveMade() {
     if (AnimateMoves)
       SoundStack.push(9);
     else
-      sndevents[9].safePlay();
+      sndevents[9].play();
   }
 }
 
 void Global::playOther(int i) {
   if (i>=N_SOUND_EVENTS) return;
   if (EnableSounds && sndevents[i].enabled)
-    sndevents[i].safePlay();
+    sndevents[i].play();
 }
 
 void Global::repaintAllBoards() {
@@ -784,7 +775,7 @@ void Global::setPasswordMode(int pm) {
     (*i)->setPasswordMode(pm);
 }
 
-void Global::debug(char *klass,char *method,char *data) {
+void Global::debug(const char *klass,const char *method,const char *data) {
   char z[256];
   time_t now;
   string rm;
@@ -813,8 +804,9 @@ void Global::debug(char *klass,char *method,char *data) {
   f.close();
 }
 
-void Global::LogAppend(char *msg) {
-  char z[256],*p;
+void Global::LogAppend(const char *msg) {
+  char z[256];
+  const char *p;
   static char hexa[17]="0123456789abcdef";
   time_t now;
   string s;
@@ -922,11 +914,11 @@ Notebook * Global::getNotebook() {
   return(ebook);
 }
 
-char * Global::filter(char *s) {
+const char * Global::filter(const char *s) {
   int i,j;
   string t;
   gunichar uc;
-  char *c;
+  const char *c;
 
   if (SpecialChars==0) return s; // no filtering
 
@@ -953,9 +945,16 @@ char * Global::filter(char *s) {
 }
 
 void Global::unicodeNormalize(string &dest, gunichar src) {
-  gunichar *tmp;
+  gunichar *tmp, tmpbuf[16];
   gsize i,len;
+
+#if (GLIB_MAJOR_VERSION==2 && GLIB_MINOR_VERSION >= 30)
+  len = g_unichar_fully_decompose(src, FALSE, tmpbuf, 16);
+  tmp = &tmpbuf[0];
+#else
   tmp = g_unicode_canonical_decomposition(src, &len);
+#endif
+
   for(i=0;i<len;i++) {
     if (tmp[i] > 128) {
       switch(tmp[i]) {
@@ -975,7 +974,9 @@ void Global::unicodeNormalize(string &dest, gunichar src) {
     dest.append( 1, (char) (tmp[i]&0x7f) );
   }
 
-  g_free(tmp);  
+#if (!(GLIB_MAJOR_VERSION==2 && GLIB_MINOR_VERSION >= 30))
+  g_free(tmp);
+#endif
 }
 
 void Global::gatherConsoleState() {
@@ -1068,7 +1069,7 @@ int EngineBookmark::operator==(EngineBookmark *ebm) {
 }
 
 void EngineBookmark::read(tstring &t) {
-  static char *sep="^\n\r";
+  static const char *sep="^\n\r";
   string *p;
 
   caption     = *(t.token(sep));
@@ -1092,7 +1093,7 @@ void EngineBookmark::read(tstring &t) {
 // -------------------------------- channel splitting
 
 IcsChannel::IcsChannel(char *s) {
-  static char *sep="\t\r\n";
+  static const char *sep="\t\r\n";
   tstring t;
   t.set(s);
   number = t.tokenvalue(sep);
@@ -1122,7 +1123,7 @@ void ChannelSplitter::getChannels(char *ipaddr) {
       goto cs_gc_use_current;
   }
 
-  snprintf(url,512,"http://eboard.sourceforge.net/ics/%s.txt",ipaddr);
+  snprintf(url,512,"http://www.bergo.eng.br/eboard/ics/%s.txt",ipaddr);
 
   kid=fork();
   if (kid==0) {
@@ -1291,7 +1292,7 @@ TerminalColor::TerminalColor() {
 }
 
 void TerminalColor::read(tstring &t) {
-  static char *comma=",:\n\r \t";  
+  static const char *comma=",:\n\r \t";  
 
   TextDefault   = t.tokenvalue(comma,16);
   TextBright    = t.tokenvalue(comma,16);
@@ -1339,7 +1340,7 @@ void WindowGeometry::setNull() {
 }
 
 void WindowGeometry::read(tstring &t) {
-  static char *sep=":,()\n\t\r ";
+  static const char *sep=":,()\n\t\r ";
   X=t.tokenvalue(sep);
   Y=t.tokenvalue(sep);
   W=t.tokenvalue(sep);
@@ -1373,7 +1374,7 @@ void Desktop::clear() {
 }
 
 void Desktop::read(tstring &t) {
-  static char *sep=":,()\n\t\r ";
+  static const char *sep=":,()\n\t\r ";
   wMain.read(t);
   wGames.read(t);
   wLocal.read(t);
@@ -1393,7 +1394,7 @@ void Desktop::writeConsoles(ostream &s, const char *key) {
 void Desktop::readConsole(tstring &t) {
   WindowGeometry *wg;
   string *p,*s;
-  static char *sep="\n\r";
+  static const char *sep="\n\r";
   
   wg=new WindowGeometry();
   wg->read(t);
