@@ -1,11 +1,10 @@
-/* $Id: global.h,v 1.87 2008/02/22 07:32:17 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2008 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -224,8 +223,8 @@ class Global : public ChannelSplitter
 
   void WrappedMainIteration();
   void WrappedMainQuit();
-  void LogAppend(char *msg);
-  void debug(char *klass,char *method,char *data=0);
+  void LogAppend(const char *msg);
+  void debug(const char *klass,const char *method,const char *data=0);
 
   void dumpGames();
   void dumpBoards();
@@ -268,7 +267,7 @@ class Global : public ChannelSplitter
   void addPieceClient(PieceChangeListener *pcl);
   void removePieceClient(PieceChangeListener *pcl);
 
-  char * filter(char *s);
+  const char * filter(const char *s);
 
   // free with free() as usual
   static void * safeMalloc(int nbytes);
@@ -305,10 +304,8 @@ class Global : public ChannelSplitter
   GtkWidget          *toplevelwidget;
   BugPane            *bugpane;
   IONotificationInterface *iowatcher;
-  JoystickListener   *joycapture;
 
   StringCollection   annotator;
-  SoundSlave         sndslave;
   ZombieHunter       zombies;
 
   int                SelfInputColor;
@@ -343,6 +340,8 @@ class Global : public ChannelSplitter
   int UseVectorPieces;
   int CheckLegality;
   int SpecialChars; // 0=none 1=truncate 2=underscore 3=soft translate (a^)
+  int MsecThreshold;
+  int MsecDigits;
 
   int SplitChannels;
   int ChannelsToConsoleToo;
@@ -364,14 +363,6 @@ class Global : public ChannelSplitter
   int IcsAllObPlayed;
   int IcsAllObObserved;
 
-  int JSCursorAxis;
-  int JSBrowseAxis;
-  int JSMoveButton;
-  int JSNextTabButton;
-  int JSPrevTabButton;
-  int JSMode;
-  int JSSpeed;
-
   char P2PName[64];
 
   TerminalColor Colors;
@@ -396,7 +387,7 @@ class Global : public ChannelSplitter
   vector<string> SoundFiles;
   vector<QButton *> QuickbarButtons;
 
-  char * Version;
+  const char * Version;
   char   SystemType[64];
   char   argv0[512];
 
@@ -404,8 +395,6 @@ class Global : public ChannelSplitter
 
   string ConsoleReply;
 
-  int JoystickFD;
-
  private:
   int createDir(char *z);
   void renumberGame(ChessGame *cg,int id);
