@@ -1,11 +1,10 @@
-/* $Id: help.h,v 1.4 2002/02/13 03:53:56 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,6 +30,11 @@
 
 namespace Help {
 
+  class AboutDialog : public ModalDialog {
+  public:
+    AboutDialog();
+  };
+
   class KeysDialog : public ModalDialog {
   public:
     KeysDialog();
@@ -41,12 +45,14 @@ namespace Help {
     DebugDialog();
 
   private:
-    char *grabOutput(char *cmdline);
+    char *grabOutput(const char *cmdline);
     char pipedata[1024];
 
     friend void debugdlg_writecons(GtkWidget *w, gpointer data);
   };
 
+  void debugdlg_writecons(GtkWidget *w, gpointer data);
+
   class GettingStarted : public NonModalDialog {
   public:
     GettingStarted();
