@@ -1,11 +1,10 @@
-/* $Id: mainwindow.cc,v 1.138 2008/02/22 14:35:43 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2008 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,96 +37,92 @@
 
 #include "config.h"
 
-#ifdef HAVE_LINUX_JOYSTICK_H
-#include <linux/joystick.h>
-#endif
+#include "xpm/icon-eboard.xpm"
 
-#include "icon-eboard.xpm"
+#include "xpm/back1.xpm"
+#include "xpm/backn.xpm"
+#include "xpm/forward1.xpm"
+#include "xpm/forwardn.xpm"
+#include "xpm/movelist.xpm"
+#include "xpm/flip.xpm"
+#include "xpm/trash.xpm"
+#include "xpm/toscratch.xpm"
 
-#include "back1.xpm"
-#include "backn.xpm"
-#include "forward1.xpm"
-#include "forwardn.xpm"
-#include "movelist.xpm"
-#include "flip.xpm"
-#include "trash.xpm"
-#include "toscratch.xpm"
-
-#include "sealoff.xpm"
-#include "sealon.xpm"
+#include "xpm/sealoff.xpm"
+#include "xpm/sealon.xpm"
 
 GdkWindow * MainWindow::RefWindow=0;
 
 static GtkItemFactoryEntry mainwindow_mainmenu[] = {
-  { N_("/_Peer"),                NULL,         NULL, 0, "<Branch>" },
-  { N_("/Peer/Connect to _FICS"),NULL,GTK_SIGNAL_FUNC(peer_connect_fics),0,NULL },
-  { N_("/Peer/ICS _Bookmarks"),  NULL, NULL, FA_ICSBOOKMARKS, "<Branch>" },
-  { N_("/Peer/_Connect to Other Server..."),NULL,GTK_SIGNAL_FUNC(peer_connect_ask),0,NULL },
-  { N_("/Peer/Direct connect with _Remote eboard..."),NULL,GTK_SIGNAL_FUNC(peer_connect_p2p),0,NULL },
-  { N_("/Peer/sep4"),            NULL,           NULL, 0, "<Separator>" },
-  { N_("/Peer/Play against _Engine"),NULL, NULL, 0,"<Branch>" },
-  { N_("/Peer/Play against Engine/GNU Chess _4..."),NULL,GTK_SIGNAL_FUNC(peer_connect_gnuchess4),0, NULL },
-  { N_("/Peer/Play against Engine/_Sjeng..."),NULL,GTK_SIGNAL_FUNC(peer_connect_sjeng), 0, NULL },
-  { N_("/Peer/Play against Engine/Cr_afty..."),NULL,GTK_SIGNAL_FUNC(peer_connect_crafty), 0, NULL },
-  { N_("/Peer/Play against Engine/sep1"),NULL,NULL,0,"<Separator>" },
-  { N_("/Peer/Play against Engine/_Generic Engine..."),NULL,GTK_SIGNAL_FUNC(peer_connect_xboard), 0,NULL },
-  { N_("/Peer/Engine B_ookmarks"), NULL, NULL, FA_ENGBOOKMARKS,"<Branch>" },
-  { N_("/Peer/sep3"),            NULL,           NULL, 0, "<Separator>" },
-  { N_("/Peer/_Empty Scratch Board"), NULL,  GTK_SIGNAL_FUNC(peer_scratch_empty), 0, NULL },
-  { N_("/Peer/_Scratch Board with Initial Position"),NULL,  GTK_SIGNAL_FUNC(peer_scratch_initial), 0, NULL },
-  { N_("/Peer/sep3"),            NULL,           NULL, 0, "<Separator>" },
-  { N_("/Peer/_Disconnect"),     NULL,  GTK_SIGNAL_FUNC(peer_disconnect), 0, NULL },
-  { N_("/Peer/sep2"),            NULL,           NULL, 0, "<Separator>" },
-  { N_("/Peer/_Quit"),           NULL,  GTK_SIGNAL_FUNC(mainwindow_destroy), 0, NULL },
-  { N_("/_Game"),                NULL,         NULL, 0, "<Branch>" },
-  { N_("/Game/_Resign"),         NULL,  GTK_SIGNAL_FUNC(game_resign), 0, NULL },
-  { N_("/Game/_Offer Draw"),     NULL,  GTK_SIGNAL_FUNC(game_draw), 0, NULL },
-  { N_("/Game/_Abort"),          NULL,  GTK_SIGNAL_FUNC(game_abort), 0, NULL },
-  { N_("/Game/Ad_journ"),        NULL,  GTK_SIGNAL_FUNC(game_adjourn), 0, NULL },
-  { N_("/Game/Retract _Move"),   NULL,  GTK_SIGNAL_FUNC(game_retract), 0, NULL },
-  { N_("/_Settings"),            NULL,         NULL, 0, "<Branch>" },
-  { N_("/Settings/_Highlight Last Move"),NULL, NULL, FA_HIGHLIGHT, "<CheckItem>" },
-  { N_("/Settings/_Animate Moves"),NULL,       NULL, FA_ANIMATE,"<CheckItem>" },
-  { N_("/Settings/Pre_move"),      NULL,       NULL, FA_PREMOVE,"<CheckItem>" },
-  { N_("/Settings/Sho_w Coordinates"), NULL,   NULL, FA_COORDS,"<CheckItem>" },
-  { N_("/Settings/sep3"),          NULL,       NULL, 0, "<Separator>" },
-  { N_("/Settings/_Beep on Opponent Moves"),NULL,NULL, FA_MOVEBEEP,"<CheckItem>" },
-  { N_("/Settings/_Enable Other Sounds"),NULL,NULL,    FA_SOUND,"<CheckItem>" },
-  { N_("/Settings/sep3"),          NULL,       NULL, 0, "<Separator>" },
-  { N_("/Settings/_ICS Behavior"),   NULL,       NULL, 0, "<Branch>" },
-  { N_("/Settings/ICS Behavior/Popup Board Panes _Upon Creation"),NULL,NULL,FA_POPUP,"<CheckItem>" },
-  { N_("/Settings/ICS Behavior/Smart _Discard Observed Boards After Game Ends"),NULL,NULL,FA_SMART,"<CheckItem>" },
-  { N_("/Settings/sep3"),          NULL,       NULL, 0, "<Separator>" },
-  { N_("/Settings/Enable Legality _Checking"),NULL,NULL,FA_LEGALITY,"<CheckItem>"},
-  { N_("/Settings/sep3"),          NULL,       NULL, 0, "<Separator>" },
-  { N_("/Settings/_Vectorized Pieces (Faster Rendering)"),NULL,NULL,FA_VECTOR,"<CheckItem>" },
-  { N_("/Settings/Bitmapped Piece _Sets"),   NULL,       NULL, 0, "<Branch>" },
-  { N_("/Settings/Bitmapped Piece Sets/Load _Theme"), NULL,NULL, FA_LOADTHEME, "<Branch>" },
-  { N_("/Settings/Bitmapped Piece Sets/Load _Pieces Only"),NULL,NULL,FA_LOADPIECES,"<Branch>"},
-  { N_("/Settings/Bitmapped Piece Sets/Load _Squares Only"), NULL, NULL, FA_LOADSQUARES, "<Branch>" },
-  { N_("/Settings/sep3"),          NULL,       NULL, 0, "<Separator>" },
-  { N_("/Settings/_Preferences..."), NULL,  GTK_SIGNAL_FUNC(sett_prefs), 0, NULL },
-  { N_("/_Windows"),              NULL,         NULL,0, "<Branch>" },
-  { N_("/Windows/_Games on Server..."),NULL,  GTK_SIGNAL_FUNC(windows_games), 0, NULL },
-  { N_("/Windows/_Ads on Server..."),NULL,  GTK_SIGNAL_FUNC(windows_sough), 0, NULL },
-  { N_("/Windows/sep1"), NULL, NULL, 0, "<Separator>" },
-  { N_("/Windows/Games on _Client..."),NULL,  GTK_SIGNAL_FUNC(windows_stock), 0, NULL },
-  { N_("/Windows/sep2"), NULL, NULL, 0, "<Separator>" },
-  { N_("/Windows/_Run Script..."),  NULL,  GTK_SIGNAL_FUNC(windows_script), 0, NULL },
-  { N_("/Windows/sep3"), NULL, NULL, 0, "<Separator>" },
-  { N_("/Windows/_Detached Console"),  NULL,  GTK_SIGNAL_FUNC(windows_detached), 0, NULL },
-  { N_("/Windows/sep3"), NULL, NULL, 0, "<Separator>" },
-  { N_("/Windows/_Find Text (upwards)..."), NULL, GTK_SIGNAL_FUNC(windows_find), 0, NULL },
-  { N_("/Windows/Find _Previous"), NULL, GTK_SIGNAL_FUNC(windows_findp), 0, NULL },
-  { N_("/Windows/Save _Text Buffer..."), NULL, GTK_SIGNAL_FUNC(windows_savebuffer),0,NULL},
-  { N_("/Windows/sep3"), NULL, NULL, 0, "<Separator>" },
-  { N_("/Windows/Save Desktop _Geometry"), NULL, GTK_SIGNAL_FUNC(windows_savedesk), 0, NULL },
-  { N_("/_Help"),              NULL,         NULL,0, "<Branch>" },
-  { N_("/Help/_Getting Started"),NULL, GTK_SIGNAL_FUNC(help_starting), 0, NULL },
-  { N_("/Help/_Keys"),NULL, GTK_SIGNAL_FUNC(help_keys), 0, NULL },
-  { N_("/Help/_Debug Info"),NULL, GTK_SIGNAL_FUNC(help_debug), 0, NULL },
-  { N_("/Help/sep4"),            NULL,           NULL, 0, "<Separator>" },
-  { N_("/Help/_About eboard..."),NULL,  GTK_SIGNAL_FUNC(help_about), 0, NULL },
+  { (gchar *)N_("/_Peer"),                NULL,         NULL, 0, (gchar *)"<Branch>" },
+  { (gchar *)N_("/Peer/Connect to _FICS"),NULL,GTK_SIGNAL_FUNC(peer_connect_fics),0,NULL },
+  { (gchar *)N_("/Peer/ICS _Bookmarks"),  NULL, NULL, FA_ICSBOOKMARKS, (gchar *)"<Branch>" },
+  { (gchar *)N_("/Peer/_Connect to Other Server..."),NULL,GTK_SIGNAL_FUNC(peer_connect_ask),0,NULL },
+  { (gchar *)N_("/Peer/Direct connect with _Remote eboard..."),NULL,GTK_SIGNAL_FUNC(peer_connect_p2p),0,NULL },
+  { (gchar *)N_("/Peer/sep4"),            NULL,           NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Peer/Play against _Engine"),NULL, NULL, 0,(gchar *)"<Branch>" },
+  { (gchar *)N_("/Peer/Play against Engine/GNU Chess _4..."),NULL,GTK_SIGNAL_FUNC(peer_connect_gnuchess4),0, NULL },
+  { (gchar *)N_("/Peer/Play against Engine/_Sjeng..."),NULL,GTK_SIGNAL_FUNC(peer_connect_sjeng), 0, NULL },
+  { (gchar *)N_("/Peer/Play against Engine/Cr_afty..."),NULL,GTK_SIGNAL_FUNC(peer_connect_crafty), 0, NULL },
+  { (gchar *)N_("/Peer/Play against Engine/sep1"),NULL,NULL,0,(gchar *)"<Separator>" },
+  { (gchar *)N_("/Peer/Play against Engine/_Generic Engine..."),NULL,GTK_SIGNAL_FUNC(peer_connect_xboard), 0,NULL },
+  { (gchar *)N_("/Peer/Engine B_ookmarks"), NULL, NULL, FA_ENGBOOKMARKS,(gchar *)"<Branch>" },
+  { (gchar *)N_("/Peer/sep3"),            NULL,           NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Peer/_Empty Scratch Board"), NULL,  GTK_SIGNAL_FUNC(peer_scratch_empty), 0, NULL },
+  { (gchar *)N_("/Peer/_Scratch Board with Initial Position"),NULL,  GTK_SIGNAL_FUNC(peer_scratch_initial), 0, NULL },
+  { (gchar *)N_("/Peer/sep3"),            NULL,           NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Peer/_Disconnect"),     NULL,  GTK_SIGNAL_FUNC(peer_disconnect), 0, NULL },
+  { (gchar *)N_("/Peer/sep2"),            NULL,           NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Peer/_Quit"),           NULL,  GTK_SIGNAL_FUNC(mainwindow_destroy), 0, NULL },
+  { (gchar *)N_("/_Game"),                NULL,         NULL, 0, (gchar *)"<Branch>" },
+  { (gchar *)N_("/Game/_Resign"),         NULL,  GTK_SIGNAL_FUNC(game_resign), 0, NULL },
+  { (gchar *)N_("/Game/_Offer Draw"),     NULL,  GTK_SIGNAL_FUNC(game_draw), 0, NULL },
+  { (gchar *)N_("/Game/_Abort"),          NULL,  GTK_SIGNAL_FUNC(game_abort), 0, NULL },
+  { (gchar *)N_("/Game/Ad_journ"),        NULL,  GTK_SIGNAL_FUNC(game_adjourn), 0, NULL },
+  { (gchar *)N_("/Game/Retract _Move"),   NULL,  GTK_SIGNAL_FUNC(game_retract), 0, NULL },
+  { (gchar *)N_("/_Settings"),            NULL,         NULL, 0, (gchar *)"<Branch>" },
+  { (gchar *)N_("/Settings/_Highlight Last Move"),NULL, NULL, FA_HIGHLIGHT, (gchar *)"<CheckItem>" },
+  { (gchar *)N_("/Settings/_Animate Moves"),NULL,       NULL, FA_ANIMATE,(gchar *)"<CheckItem>" },
+  { (gchar *)N_("/Settings/Pre_move"),      NULL,       NULL, FA_PREMOVE,(gchar *)"<CheckItem>" },
+  { (gchar *)N_("/Settings/Sho_w Coordinates"), NULL,   NULL, FA_COORDS,(gchar *)"<CheckItem>" },
+  { (gchar *)N_("/Settings/sep3"),          NULL,       NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Settings/_Beep on Opponent Moves"),NULL,NULL, FA_MOVEBEEP,(gchar *)"<CheckItem>" },
+  { (gchar *)N_("/Settings/_Enable Other Sounds"),NULL,NULL,    FA_SOUND,(gchar *)"<CheckItem>" },
+  { (gchar *)N_("/Settings/sep3"),          NULL,       NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Settings/_ICS Behavior"),   NULL,       NULL, 0, (gchar *)"<Branch>" },
+  { (gchar *)N_("/Settings/ICS Behavior/Popup Board Panes _Upon Creation"),NULL,NULL,FA_POPUP,(gchar *)"<CheckItem>" },
+  { (gchar *)N_("/Settings/ICS Behavior/Smart _Discard Observed Boards After Game Ends"),NULL,NULL,FA_SMART,(gchar *)"<CheckItem>" },
+  { (gchar *)N_("/Settings/sep3"),          NULL,       NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Settings/Enable Legality _Checking"),NULL,NULL,FA_LEGALITY,(gchar *)"<CheckItem>"},
+  { (gchar *)N_("/Settings/sep3"),          NULL,       NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Settings/_Vectorized Pieces (Faster Rendering)"),NULL,NULL,FA_VECTOR,(gchar *)"<CheckItem>" },
+  { (gchar *)N_("/Settings/Bitmapped Piece _Sets"),   NULL,       NULL, 0, (gchar *)"<Branch>" },
+  { (gchar *)N_("/Settings/Bitmapped Piece Sets/Load _Theme"), NULL,NULL, FA_LOADTHEME, (gchar *)"<Branch>" },
+  { (gchar *)N_("/Settings/Bitmapped Piece Sets/Load _Pieces Only"),NULL,NULL,FA_LOADPIECES,(gchar *)"<Branch>"},
+  { (gchar *)N_("/Settings/Bitmapped Piece Sets/Load _Squares Only"), NULL, NULL, FA_LOADSQUARES, (gchar *)"<Branch>" },
+  { (gchar *)N_("/Settings/sep3"),          NULL,       NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Settings/_Preferences..."), NULL,  GTK_SIGNAL_FUNC(sett_prefs), 0, NULL },
+  { (gchar *)N_("/_Windows"),              NULL,         NULL,0, (gchar *)"<Branch>" },
+  { (gchar *)N_("/Windows/_Games on Server..."),NULL,  GTK_SIGNAL_FUNC(windows_games), 0, NULL },
+  { (gchar *)N_("/Windows/_Ads on Server..."),NULL,  GTK_SIGNAL_FUNC(windows_sough), 0, NULL },
+  { (gchar *)N_("/Windows/sep1"), NULL, NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Windows/Games on _Client..."),NULL,  GTK_SIGNAL_FUNC(windows_stock), 0, NULL },
+  { (gchar *)N_("/Windows/sep2"), NULL, NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Windows/_Run Script..."),  NULL,  GTK_SIGNAL_FUNC(windows_script), 0, NULL },
+  { (gchar *)N_("/Windows/sep3"), NULL, NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Windows/_Detached Console"),  NULL,  GTK_SIGNAL_FUNC(windows_detached), 0, NULL },
+  { (gchar *)N_("/Windows/sep3"), NULL, NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Windows/_Find Text (upwards)..."), NULL, GTK_SIGNAL_FUNC(windows_find), 0, NULL },
+  { (gchar *)N_("/Windows/Find _Previous"), NULL, GTK_SIGNAL_FUNC(windows_findp), 0, NULL },
+  { (gchar *)N_("/Windows/Save _Text Buffer..."), NULL, GTK_SIGNAL_FUNC(windows_savebuffer),0,NULL},
+  { (gchar *)N_("/Windows/sep3"), NULL, NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Windows/Save Desktop _Geometry"), NULL, GTK_SIGNAL_FUNC(windows_savedesk), 0, NULL },
+  { (gchar *)N_("/_Help"),              NULL,         NULL,0, (gchar *)"<Branch>" },
+  { (gchar *)N_("/Help/_Getting Started"),NULL, GTK_SIGNAL_FUNC(help_starting), 0, NULL },
+  { (gchar *)N_("/Help/_Keys"),NULL, GTK_SIGNAL_FUNC(help_keys), 0, NULL },
+  { (gchar *)N_("/Help/_Debug Info"),NULL, GTK_SIGNAL_FUNC(help_debug), 0, NULL },
+  { (gchar *)N_("/Help/sep4"),            NULL,           NULL, 0, (gchar *)"<Separator>" },
+  { (gchar *)N_("/Help/_About eboard..."),NULL,  GTK_SIGNAL_FUNC(help_about), 0, NULL },
 
 };  
 
@@ -154,7 +149,6 @@ MainWindow::MainWindow() {
   consolecopy=0;
   scriptlist=0;
   ims=0;
-  jpd = 0;
 
   io_tag = -1;
 
@@ -543,7 +537,7 @@ void MainWindow::createNavbar(GtkWidget *box) {
 		     GTK_SIGNAL_FUNC(navbar_toscratch),this);
 }
 
-void MainWindow::setTitle(char *msg) {
+void MainWindow::setTitle(const char *msg) {
   gtk_window_set_title(GTK_WINDOW(widget),msg);
 }
 
@@ -569,7 +563,7 @@ void MainWindow::restoreDesk() {
 }
 
 void MainWindow::parseThemeFile(char *name) {
-  static char * comma = ",";
+  static const char * comma = ",";
   char s[256], aux[16];
   ThemeEntry *te;
   list<ThemeEntry *>::iterator it;
@@ -789,11 +783,11 @@ void MainWindow::greet() {
   char z[128];
   snprintf (z,128,_("eboard version %s (%s)"),global.Version,global.SystemType);
   icsout->append(z,0xc0ff00,IM_IGNORE);
-  snprintf (z,128,_("(c) 2000-%d Felipe Bergo <fbergo@gmail.com> (FICS handle: Pulga)"), 2007);
+  snprintf (z,128,_("(c) 2000-%d Felipe Bergo <fbergo@gmail.com> (FICS handle: Pulga)"), 2016);
   icsout->append(z,0xc0ff00,IM_IGNORE);
   icsout->append(_("Distributed under the terms of the GNU General Public License, version 2 or later"),0xffc000,IM_IGNORE);
   icsout->append("http://www.gnu.org/copyleft/gpl.html",0xffc000,IM_IGNORE);
-  icsout->append(_("Source code available at http://eboard.sourceforge.net"),0xc0ff00,IM_IGNORE);
+  icsout->append(_("Source code available at https://github.com/fbergo/eboard"),0xc0ff00,IM_IGNORE);
   icsout->append("---",0xc0ff00,IM_IGNORE);
 
 }
@@ -967,8 +961,8 @@ void MainWindow::saveDesk() {
   global.writeRC();  
 }
 
-void MainWindow::openServer(char *host,int port,Protocol *protocol,
-			    char *helper) {
+void MainWindow::openServer(const char *host,int port,Protocol *protocol,
+			    const char *helper) {
   global.debug("MainWindow","openServer",host);
   inconsole->pop();
   tryConnect(host,port,protocol,helper);
@@ -1059,8 +1053,8 @@ void MainWindow::openEngine(EngineProtocol *xpp, EngineBookmark *ebm) {
   }
 }
 
-void MainWindow::tryConnect(char *host,int port,Protocol *protocol,
-			    char *helper)
+void MainWindow::tryConnect(const char *host,int port,Protocol *protocol,
+			    const char *helper)
 {
   DirectConnection *bytcp;
   PipeConnection *bypipe;
@@ -1274,53 +1268,6 @@ gboolean do_smart_remove(gpointer data) {
   return FALSE;
 }
 
-void MainWindow::joystickEvent(JoystickEventType jet, int number, int value) {
-  int id, anum;
-  ChessGame *cg = NULL;
-  Board *b = NULL;
-
-  if (global.joycapture != NULL) {
-    global.joycapture->joystickEvent(jet,number,value);
-    return;
-  }
-  
-  id=notebook->getCurrentPageId();
-
-  if (id==-1) 
-    b=global.BoardList.front();
-  else {
-    cg=global.getGame(id);
-    if (cg!=NULL)
-      b=cg->getBoard();
-  }
-
-  anum = number & 0xfffe;
-
-  if (jet == JOY_AXIS) {
-    if (anum == global.JSCursorAxis && b!=NULL && 
-	(global.JSMode==1 || value<-16000 || value>16000 || value==0) )
-      b->joystickCursor(number%2, value);
-    if (number == global.JSBrowseAxis && b!=NULL && (value<-16000 || value>16000 || value==0) ) {
-      if (value < 0) value = -1;
-      if (value > 0) value =  1;
-
-      if (value < 0 && jpd!=value) b->walkBack1();
-      if (value > 0 && jpd!=value) b->walkForward1();
-      jpd = value;
-    }
-  }
-
-  if (jet == JOY_BUTTON && value==1) {
-    if (number == global.JSMoveButton && b!=NULL)
-      b->joystickSelect();
-    if (number == global.JSPrevTabButton)
-      global.ebook->goToPrevious();
-    if (number == global.JSNextTabButton)
-      global.ebook->goToNext();
-  }
-
-}
-
 // changes enabling of buttons upon pane change
 // also schedules 'smart removes' for finished boards
 void MainWindow::paneChanged(int pgseq,int pgid) {
@@ -1435,6 +1382,13 @@ main_key_press (GtkWidget * wid, GdkEventKey * evt, gpointer data) {
     case GDK_g:
       windows_findp(0,0);
       break;
+    case GDK_T:
+    case GDK_t:
+      global.ShowTimestamp = (global.ShowTimestamp ? 0 : 1);
+      global.output->updateFont();
+      global.updateFont();
+      global.writeRC();
+      break;
     default:
       return 0;
     }
@@ -2035,7 +1989,7 @@ string & InputModeSelector::getPrefix() {
   return prefix;
 }
 
-void InputModeSelector::setPrefix(char *s) {
+void InputModeSelector::setPrefix(const char *s) {
   prefix=s;
   if (ChatMode) setChatMode(true);
 }
@@ -2105,16 +2059,3 @@ void PrefixCache::setIfNotSet(int id, string &val) {
   ids.push_back(id);
   text.push_back( new string(val) );
 }
-
-#ifdef HAVE_LINUX_JOYSTICK_H
-void mainwindow_joystick(gpointer data,gint source,GdkInputCondition cond) {
-  struct js_event jse;
-  MainWindow *me = (MainWindow *) data;
-  if (read(global.JoystickFD,&jse,sizeof(struct js_event)) == sizeof(struct js_event)) {
-    switch(jse.type) {
-    case JS_EVENT_BUTTON: me->joystickEvent(JOY_BUTTON, (int) (jse.number), (int) (jse.value)); break;
-    case JS_EVENT_AXIS:   me->joystickEvent(JOY_AXIS, (int) (jse.number), (int) (jse.value)); break;      
-    }
-  }
-}
-#endif
