@@ -1,11 +1,10 @@
-/* $Id: movelist.cc,v 1.18 2007/01/20 15:58:43 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,7 +31,7 @@
 #include "stl.h"
 #include "eboard.h"
 
-#include "icon-moves.xpm"
+#include "xpm/icon-moves.xpm"
 
 MoveListWindow::MoveListWindow(char *p1,char *p2,
 			       int gid,list<Position> &moves,
