@@ -1,11 +1,10 @@
-/* $Id: network.cc,v 1.42 2003/11/15 03:17:58 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,6 +38,41 @@
 #include "config.h"
 #include "eboard.h"
 
+
+// pipewatch
+
+gint pw_toid = -1;
+#define PW_MAX 16
+static PipeConnection *pw_array[PW_MAX];
+static int pw_count = 0;
+
+gboolean pipewatch_handler(gpointer data) {
+  int i;
+  for(i=pw_count-1;i>=0;i--) {
+    PipeConnection *pc = pw_array[i];
+    if (pc!=NULL) {
+      pc->consume(pc->pin,1536);
+      // wait for data on the pipe before shutting down
+      pc->toid=gtk_timeout_add(750,sched_close,(gpointer)pc);
+      pw_array[i] = NULL;
+      pw_count--;
+    }
+  }
+  return TRUE;
+}
+
+void pipewatch_start() {
+  if (pw_toid < 0)
+    pw_toid = gtk_timeout_add(250,pipewatch_handler,NULL);
+}
+
+void pipewatch_end() {
+  if (pw_toid >= 0) {
+    gtk_timeout_remove(pw_toid);
+    pw_toid = -1;
+  }
+}
+
 // ===================================================================
 // PROCESS CONTROL
 // ===================================================================
@@ -63,7 +97,7 @@ void PidRing::remove(PidIssuer *po) {
     if ( (*i)->issuer == po ) {
        x = *i;
       parents.erase(i);
-      delete(x);
+      //delete(x);
       return;
     }
 }
@@ -107,6 +141,12 @@ NetConnection::~NetConnection() {
   notifyReadReady(NULL);
 }
 
+void NetConnection::cleanUp() {
+  if (TagRead>=0)
+    gdk_input_remove(TagRead);
+  TagRead = -1;
+}
+
 void NetConnection::notifyReadReady(IONotificationInterface *target) {
   if (TagRead>=0)
     gdk_input_remove(TagRead);
@@ -136,34 +176,106 @@ int  NetConnection::hasTimeGuard() {
   return TimeGuard;
 }
 
+
+RingBuffer::RingBuffer(int _size) {
+  size = _size;
+  rp=wp=len=0;
+  data = (char *) malloc(size);
+}
+
+RingBuffer::~RingBuffer() {
+  if (data!=NULL) free(data);
+}
+
+bool RingBuffer::empty() const { return(len==0); }
+
+bool RingBuffer::full() const { return(len==size); }
+
+char RingBuffer::pop() {
+  char c;
+  if (len==0) return -1;
+  c = data[rp];
+  rp = (rp+1)%size;
+  --len;
+  return c;
+}
+
+bool RingBuffer::push(char c) {
+  if (len==size) return false;
+  data[wp] = c;
+  wp = (wp+1)%size;
+  ++len;
+  return true;
+}
+
+char RingBuffer::last() const {
+  char c;
+  if (len==0) return -1;
+  c = data[(rp+len-1)%size];
+  return c;
+}
+
+string RingBuffer::toString(int maxlen) const {
+  string s;
+  int i;
+  if (maxlen < 0) maxlen = len;
+
+  for(i=0;i<maxlen;i++)
+    s.append(1,data[(rp+i)%size]);
+
+  return s;
+}
+
+string RingBuffer::getLine() {
+  string s;
+  int i,j;
+
+  for(i=0;i<len;i++)
+    if (data[(rp+i)%size]=='\n')
+      break;
+
+  if (i<len) {
+    ++i;
+    for(j=0;j<i;j++)
+      s.append(1,pop());
+  }
+  return s;
+}
+
+void RingBuffer::status() {
+  int i;
+  char c;
+  printf("size=%d len=%d rp=%d wp=%d\n[",size,len,rp,wp);
+  for(i=0;i<len;i++) {
+    c = data[(rp+i)%size];
+    if (isprint(c)) putchar(c); else printf("{%.2x}",c);
+  }
+  printf("]\n");
+}
+
 // ===================================================================
 // BUFFERED
 // ===================================================================
 
 int BufferedConnection::readPartial(char *tbuffer,int limit) {
   if (buffer.empty()) return -1;
+  char c;
   int i;
   memset(tbuffer,0,limit);
   for(i=0;!buffer.empty();) {
-    if (buffer.front()>=32)
-      tbuffer[i++]=buffer.front();
-    buffer.pop_front();
+    c = buffer.pop();
+    if (c>=32)
+      tbuffer[i++]=c;
   }
   return 0;  
 }
 
-int BufferedConnection::bufferMatch(char *match) {
-  char dump[512];
-  list<char>::iterator li;
+bool BufferedConnection::bufferMatch(const char *match) {
+  string peek;
   int i;
-  if (buffer.empty()) return 0;  
-  memset(dump,0,512);
-  for(i=0,li=buffer.begin();li!=buffer.end();li++,i++) {
-    dump[i]=*li;
-    if (i>510)
-      return 0;
-  }
-  return(strstr(dump,match)!=0);
+  if (buffer.empty()) return false;
+  peek = buffer.toString(512);
+  return(peek.find(string(match))!=string::npos);
 }
 
 int BufferedConnection::consume(int handle, int amount) {
@@ -171,7 +283,9 @@ int BufferedConnection::consume(int handle, int amount) {
   char sm[2048];
   if (amount>2048) amount=2048;
   //  global.debug("I/O","consume-in");
+  
   while(1) {
+    errno = 0;
     i=read(handle,sm,amount);
     if ((i==0)&&(errno==0)) {
       if (buffer.empty()) {
@@ -179,15 +293,18 @@ int BufferedConnection::consume(int handle, int amount) {
 	//	global.debug("I/O","consume-out");
 	return -1;
       } else {
-	if (buffer.back()!='\n')
-	  buffer.push_back('\n');
+	if (buffer.last()!='\n') {
+	  //buffer.status();
+	  buffer.push('\n');
+	}
 	break;
       }
     }
-    if (i<=0)
+    if (i<=0) {
       break;
+    }
     for(j=0;j<i;j++) {
-      buffer.push_back(sm[j]);
+      buffer.push(sm[j]);
       // cerr << sm[j] << flush;
     }
   }
@@ -197,23 +314,20 @@ int BufferedConnection::consume(int handle, int amount) {
 
 int BufferedConnection::produce(char *tbuffer,int limit,int handle) {
   int i;
+  unsigned int j;
   char c;
-  list<char>::iterator di;
+  string s;
 
-  for(di=buffer.begin();di!=buffer.end();di++)
-    if (*di=='\n')
-      break;
+  s = buffer.getLine();
 
-  if (di!=buffer.end()) {
-    memset(tbuffer,0,limit);
+  if (!s.empty()) {
     i=0;
-    while(di!=buffer.begin()) {
-      c=buffer.front();
-      buffer.pop_front();
-      if (c>=0x20)
-	tbuffer[i++]=c;
+    memset(tbuffer,0,limit);
+    for(j=0;j<s.size();j++) {
+      if (i<limit-1 && s[j]>=32) {
+	tbuffer[i++] = s[j];
+      }
     }
-    buffer.pop_front();
     global.LogAppend(tbuffer);
     return 0;
   }
@@ -226,86 +340,16 @@ int BufferedConnection::innerReadLine(char *tbuffer,int limit,int handle) {
   return(produce(tbuffer,limit,handle));
 }
 
-int BufferedConnection::bufferEmpty() {
-  list<char>::iterator di;
-  for(di=buffer.begin();di!=buffer.end();di++)
-    if (*di=='\n')
-      return 0;
-  return 1;
-}
-
-// ===================================================================
-// ALTBUFFERED
-// ===================================================================
-
-int AltBufferedConnection::innerReadLine(char *tbuffer,int limit,int handle) {
-  int i,j;
-  char sm[128];
-  char c;
-  list<char>::iterator di;
-
-  fd_set mine,*mp;
-  struct timeval tv; 
-  mp=&mine;
-
-  while(1) {
-    // do the select thing...
-    FD_ZERO(mp);
-    FD_SET(handle,mp);
-    tv.tv_sec=0;
-    tv.tv_usec=20000;
-
-    if (select(handle+1,mp,0,0,&tv)<=0)
-      break;
-
-    i=read(handle,sm,128);
-    if (i<=0) {
-      if (buffer.empty()) {
-	close();
-	return -1;
-      } else {
-	if (buffer.back()!='\n')
-	  buffer.push_back('\n');
-	break;
-      }
-    }
-    for(j=0;j<i;j++) {
-      buffer.push_back(sm[j]);
-      // cerr << sm[j] << flush;
-    }
-  }
-
-  for(di=buffer.begin();di!=buffer.end();di++)
-    if (*di=='\n')
-      break;
-
-  if (di!=buffer.end()) {
-    memset(tbuffer,0,limit);
-    i=0;
-    while(di!=buffer.begin()) {
-      c=buffer.front();
-      buffer.pop_front();
-      if (c>=0x20)
-	tbuffer[i++]=c;
-    }
-    global.LogAppend(tbuffer);
-    buffer.pop_front();
-    return 0;
-  }
-
-  return -1;
-}
-
 // ===================================================================
 // DIRECT
 // ===================================================================
 
-DirectConnection::DirectConnection(char *hostname,int port) {
+DirectConnection::DirectConnection(const char *hostname,int port) {
   strcpy(HostName,hostname);
   strcpy(HostAddress,"???");
   Port=port;
   Connected=0;
-  strcpy(errorMessage,_("No error."));
+  g_strlcpy(errorMessage,_("No error."),128);
 }
 
 int DirectConnection::open() {
@@ -314,24 +358,24 @@ int DirectConnection::open() {
 
   if (global.CommLog) {
     char ls[512];
-    sprintf(ls,"+ DirectConnection::open(%s,%d)",HostName,Port);
+    snprintf(ls,512,"+ DirectConnection::open(%s,%d)",HostName,Port);
     global.LogAppend(ls);
   }
 
-  sprintf(z,_("Looking up host %s..."),HostName);
+  snprintf(z,128,_("Looking up host %s..."),HostName);
   global.status->setText(z,30);
   
   he=gethostbyname(HostName);
   if (he==NULL) {
-    sprintf(errorMessage,_("Host not found: %s"),HostName);
+    snprintf(errorMessage,128,_("Host not found: %s"),HostName);
     return(-1);
   }
 
-  sprintf(HostAddress,"%d.%d.%d.%d",
-	  (guchar) he->h_addr_list[0][0],
-	  (guchar) he->h_addr_list[0][1],
-	  (guchar) he->h_addr_list[0][2],
-	  (guchar) he->h_addr_list[0][3]);
+  snprintf(HostAddress,96,"%d.%d.%d.%d",
+	   (guchar) he->h_addr_list[0][0],
+	   (guchar) he->h_addr_list[0][1],
+	   (guchar) he->h_addr_list[0][2],
+	   (guchar) he->h_addr_list[0][3]);
 
   netsocket=socket(AF_INET,SOCK_STREAM,0);
 
@@ -349,32 +393,32 @@ int DirectConnection::open() {
   sa.sin_port=htons(Port);
   memcpy(&sa.sin_addr,he->h_addr_list[0],he->h_length);
 
-  sprintf(z,_("Connecting to %s..."),HostAddress);
+  snprintf(z,128,_("Connecting to %s..."),HostAddress);
   global.status->setText(z,30);
 
   i=::connect(netsocket,(struct sockaddr *)&sa,sizeof(sa));
 
   if (i!=0) {
-    sprintf(z,_("Connection to %s:%d failed: "),HostName,Port);
+    snprintf(z,128,_("Connection to %s:%d failed: "),HostName,Port);
     switch(errno) {
-    case EBADF:        strcat(z,_("Bad descriptor"));         break;
-    case EFAULT:       strcat(z,_("Wrong address space"));    break;
-    case ENOTSOCK:     strcat(z,_("Not a socket ?!?"));       break;
-    case EISCONN:      strcat(z,_("Already connected ?!?"));  break;
-    case ECONNREFUSED: strcat(z,_("Connection refused"));     break;
-    case ETIMEDOUT:    strcat(z,_("Timeout"));                break;
-    case ENETUNREACH:  strcat(z,_("Network is unreachable")); break;
-    case EADDRINUSE:   strcat(z,_("Address already in use")); break;
+    case EBADF:        g_strlcat(z,_("Bad descriptor"),128);         break;
+    case EFAULT:       g_strlcat(z,_("Wrong address space"),128);    break;
+    case ENOTSOCK:     g_strlcat(z,_("Not a socket ?!?"),128);       break;
+    case EISCONN:      g_strlcat(z,_("Already connected ?!?"),128);  break;
+    case ECONNREFUSED: g_strlcat(z,_("Connection refused"),128);     break;
+    case ETIMEDOUT:    g_strlcat(z,_("Timeout"),128);                break;
+    case ENETUNREACH:  g_strlcat(z,_("Network is unreachable"),128); break;
+    case EADDRINUSE:   g_strlcat(z,_("Address already in use"),128); break;
     case EALREADY:
     case EINPROGRESS:  strcat(z,"EINPROGRESS");           break;
-    default:           strcat(z,_("Unknown error"));
+    default:           g_strlcat(z,_("Unknown error"),128);
     }
-    strcpy(errorMessage,z);
+    g_strlcpy(errorMessage,z,128);
     return(-1);
   }
   Connected=1;
   fcntl(netsocket,F_SETFL,O_NONBLOCK);
-  sprintf(z,_("Connected to %s (%s)"),HostName,HostAddress);
+  snprintf(z,128,_("Connected to %s (%s)"),HostName,HostAddress);
   global.status->setText(z,300);
 
   global.getChannels(HostAddress);
@@ -392,15 +436,18 @@ int DirectConnection::readLine(char *tbuffer,int limit) {
 }
 
 
-void DirectConnection::writeLine(char *obuffer) {
+void DirectConnection::writeLine(const char *obuffer) {
   if (Connected) {
-    write(netsocket,obuffer,strlen(obuffer));
+    const char *tmp;
+    tmp = global.filter(obuffer);
+    write(netsocket,tmp,strlen(tmp));
     write(netsocket,"\n",1);
     if (global.CommLog) {
       char z[4096];
-      sprintf(z,"WROTE: %s",obuffer);
+      snprintf(z,4096,"WROTE: %s",tmp);
       global.LogAppend(z);
     }
+    if (tmp != obuffer) free((void *)tmp);
   }
 }
 
@@ -426,7 +473,7 @@ IncomingConnection::IncomingConnection(int port) {
   Port = port;
   Connected = 0;
   netsocket = 0;  
-  strcpy(errorMessage,_("No error."));
+  g_strlcpy(errorMessage,_("No error."),128);
 }
 
 /* the first open call on this object only
@@ -445,7 +492,7 @@ int IncomingConnection::createSocket() {
 
   netsocket = socket(AF_INET, SOCK_STREAM, 0);
   if (netsocket == -1) {
-    strcpy(errorMessage,_("Unable to create socket."));
+    g_strlcpy(errorMessage,_("Unable to create socket."),128);
     return -1;
   }
 
@@ -455,13 +502,13 @@ int IncomingConnection::createSocket() {
   sin.sin_port = htons(Port);
     
   if (bind(netsocket,(struct sockaddr *) &sin,sizeof(sin))==-1) {
-    sprintf(errorMessage,_("Unable to bind on port %d."),Port);
+    snprintf(errorMessage,128,_("Unable to bind on port %d."),Port);
     return(-1);
   }
   
   fcntl(netsocket,F_SETFL,O_NONBLOCK);
   if (listen(netsocket, 1)!=0) {
-    sprintf(errorMessage,"Unable to listen on port %d.",Port);
+    snprintf(errorMessage,128,"Unable to listen on port %d.",Port);
     return(-1);
   }
   return 0;
@@ -485,17 +532,17 @@ int IncomingConnection::acceptConnection() {
     return -1;
   }
 
-  strcpy(HostName, inet_ntoa(pin.sin_addr));
+  g_strlcpy(HostName, inet_ntoa(pin.sin_addr),128);
   strcpy(HostAddress, HostName);
 
   ::close(netsocket); /* kill the listening socket */
   netsocket = sock;
 
-  strcpy(errorMessage,_("No error."));
+  g_strlcpy(errorMessage,_("No error."),128);
   Connected = 1;
 
   fcntl(netsocket,F_SETFL,O_NONBLOCK);
-  sprintf(z,_("Accepted incoming connection from %s"),HostName);
+  snprintf(z,128,_("Accepted incoming connection from %s"),HostName);
   global.status->setText(z,300);
 
   return 0;
@@ -517,15 +564,18 @@ int IncomingConnection::readLine(char *tbuffer, int limit) {
   return(Connected?innerReadLine(tbuffer,limit,netsocket):-1);
 }
 
-void IncomingConnection::writeLine(char *obuffer) {
+void IncomingConnection::writeLine(const char *obuffer) {
   if (Connected) {
-    write(netsocket,obuffer,strlen(obuffer));
+    const char *tmp;
+    tmp = global.filter(obuffer);
+    write(netsocket,tmp,strlen(tmp));
     write(netsocket,"\n",1);
     if (global.CommLog) {
       char z[4096];
-      sprintf(z,"WROTE: %s",obuffer);
+      snprintf(z,4096,"WROTE: %s",tmp);
       global.LogAppend(z);
     }
+    if (tmp!=obuffer) free((void *)tmp);
   }
 }
 
@@ -552,8 +602,9 @@ PipeConnection::PipeConnection(int _pin,int _pout) {
   fcntl(pin,F_SETFL,O_NONBLOCK);
   Connected=1;
   strcpy(HostName,"local pipe");
-  sprintf(HostAddress,"pipe[%d,%d]",pin,pout);
+  snprintf(HostAddress,96,"pipe[%d,%d]",pin,pout);
   Quiet=0;
+  MaxWaitTime = 60000.0; // 1 minute
 }
 
 void PipeConnection::init() {
@@ -565,13 +616,20 @@ void PipeConnection::init() {
   memset(HelperBin,0,512);
   Quiet=0;
   handshake.erase();
+  MaxWaitTime = 60000.0; // 1 minute
 }
 
-PipeConnection::PipeConnection(char *helperbin,char *arg1=0,char *arg2=0,
-			       char *arg3=0,char *arg4=0) {
+void PipeConnection::setMaxWaitTime(double msecs) {
+  MaxWaitTime = msecs;
+}
+
+PipeConnection::PipeConnection(const char *helperbin,const char *arg1,
+			       const char *arg2,
+			       const char *arg3, 
+			       const char *arg4) {
   init();
   opmode=0;
-  strncpy(HelperBin,helperbin,511);
+  g_strlcpy(HelperBin,helperbin,512);
   args.push_back(HelperBin);
   if (arg1) args.push_back(arg1);
   if (arg2) args.push_back(arg2);
@@ -579,12 +637,12 @@ PipeConnection::PipeConnection(char *helperbin,char *arg1=0,char *arg2=0,
   if (arg4) args.push_back(arg4);
 }
 
-void PipeConnection::setHandshake(char *s) {
+void PipeConnection::setHandshake(const char *s) {
   handshake=s;
 }
 
-PipeConnection::PipeConnection(char *host,int port, 
-			       char *helperbin,char *helpersuffix)
+PipeConnection::PipeConnection(const char *host,int port, 
+			       const char *helperbin,const char *helpersuffix)
 {
   char z[256];
   EboardFileFinder eff;
@@ -592,15 +650,16 @@ PipeConnection::PipeConnection(char *host,int port,
   init();
   opmode=1;
   Port=port;
-  strncpy(HostName,host,128);
+  g_strlcpy(HostName,host,128);
 
   // build helper path
   if (helpersuffix)
-    sprintf(z,"%s.%s",helperbin,helpersuffix);
+    snprintf(z,256,"%s.%s",helperbin,helpersuffix);
   else
-    strcpy(z,helperbin);
+    g_strlcpy(z,helperbin,256);
 
-  eff.find(z,HelperBin);
+  if (!eff.find(z,HelperBin))
+    HelperBin[0] = 0;
 }
 
 PipeConnection::~PipeConnection() {
@@ -617,8 +676,10 @@ int PipeConnection::isConnected() {
 
 void PipeConnection::checkChildren() {
   pid_t r;
-  if (!pid)
+  if (!pid) {
+    Connected=0;
     return;
+  }
   r=waitpid((pid_t)pid,NULL,WNOHANG);
   if (r>0)
     Connected=0;
@@ -638,39 +699,44 @@ int PipeConnection::open() {
 
   if (global.CommLog) {
     char ls[512];
-    sprintf(ls,"+ PipeConnection::open(%s)",HelperBin);
+    snprintf(ls,512,"+ PipeConnection::open(%s)",HelperBin);
     global.LogAppend(ls);
   }
 
   for(i=0;i<6;i++)
-    arguments[i]=0;
+    arguments[i]=NULL;
+
+  if (HelperBin[0]==0) {
+    snprintf(errorMessage,128,_("Helper program not found"));
+    return -1;
+  }
 
   switch(opmode) {
   case 0:
     for(i=0;i<args.size();i++)
-      arguments[i]=args[i];
+      arguments[i]= (char *) args[i];
     break;
   case 1:
-    sprintf(z,_("Looking up host %s..."),HostName);
+    snprintf(z,256,_("Looking up host %s..."),HostName);
     if (!Quiet)
       global.status->setText(z,30);
     he=gethostbyname(HostName);
     if (he==NULL) {
-      sprintf(errorMessage,_("Host not found: %s"),HostName);
+      snprintf(errorMessage,128,_("Host not found: %s"),HostName);
       return(-1);
     }
 
-    sprintf(HostAddress,"%d.%d.%d.%d",
-	    (guchar) he->h_addr_list[0][0],
-	    (guchar) he->h_addr_list[0][1],
-	    (guchar) he->h_addr_list[0][2],
-	    (guchar) he->h_addr_list[0][3]);
+    snprintf(HostAddress,96,"%d.%d.%d.%d",
+	     (guchar) he->h_addr_list[0][0],
+	     (guchar) he->h_addr_list[0][1],
+	     (guchar) he->h_addr_list[0][2],
+	     (guchar) he->h_addr_list[0][3]);
     
-    sprintf(z,_("Connecting to %s..."),HostAddress);
+    snprintf(z,256,_("Connecting to %s..."),HostAddress);
     if (!Quiet)
       global.status->setText(z,30);
 
-    sprintf(z,"%d",Port);
+    snprintf(z,256,"%d",Port);
     arguments[0]=HelperBin;
     arguments[1]=HostAddress;
     arguments[2]=z;
@@ -680,14 +746,14 @@ int PipeConnection::open() {
   // step 2: run helper
 
   if (pipe(n2h)||pipe(h2n)) {
-    strcpy(errorMessage,_("IPC pipe creation failed."));
+    g_strlcpy(errorMessage,_("IPC pipe creation failed."),128);
     return -1;    
   }
   signal(SIGPIPE,SIG_IGN);
 
   pid=fork();
   if (pid < 0) {
-    strcpy(errorMessage,_("process creation failed."));
+    g_strlcpy(errorMessage,_("process creation failed."),128);
     return -1;
   }
   if (!pid) {
@@ -719,8 +785,7 @@ int PipeConnection::open() {
       int wr,j,n;
       char hs[256];
 
-      strncpy(hs,handshake.c_str(),255);
-      hs[255]=0;
+      g_strlcpy(hs,handshake.c_str(),256);
       n=strlen(hs);
       j=0;
 
@@ -732,26 +797,53 @@ int PipeConnection::open() {
       } while(n);      
     }
 
+    fcntl(h2n[0],F_SETFL,O_NONBLOCK);
+    int rs;
+    double elapsed;
+    struct timeval ref, now;
+    gettimeofday(&ref,NULL);
+
     while(1) {
-      if (read(h2n[0],p,1)==1) {
+      rs = read(h2n[0],p,1);
+      if (rs==1) {
 	if (*p=='\n') break;
 	++p;
       }
-
+      if (rs==-1) {
+	if (errno==EAGAIN) {
+	  usleep(10000); // 10 msec
+	  gettimeofday(&now,NULL);
+	  elapsed = (1000.0*now.tv_sec - 1000.0*ref.tv_sec) + 
+	    (now.tv_usec/1000.0 - ref.tv_usec/1000.0);
+	  // MaxWaitTime msecs with no output from the process ? give up.
+	  if (elapsed > MaxWaitTime) {
+	    kill(-pid,SIGKILL);
+	    g_strlcpy(errorMessage,_("No output from program."),128);
+	    ::close(h2n[0]);
+	    ::close(n2h[1]);
+	    return -1;
+	  }
+	} else {
+	    kill(-pid,SIGKILL);
+	    g_strlcpy(errorMessage,_("Read error from program."),128);
+	    ::close(h2n[0]);
+	    ::close(n2h[1]);
+	    return -1;	  
+	}
+      } // rs=-1
       // should fix splange's crashing bug
       if ( (last_dead_pid == pid) || 
            (waitpid(pid, 0, WNOHANG) != 0) ) {
 	last_dead_pid = 0;
-	strcpy(errorMessage,_("Program exited too soon"));
+	g_strlcpy(errorMessage,_("Program exited too soon"),128);
 	::close(h2n[0]);
 	::close(n2h[1]);
 	return -1;
       }
-
     }
     *p=0;
     if (!strcmp(firstline,"exec failed")) {
-      strcpy(errorMessage,_("Failed to run helper program"));
+      g_strlcpy(errorMessage,_("Failed to run helper program"),128);
       ::close(h2n[0]);
       ::close(n2h[1]);
       return -1;
@@ -767,9 +859,9 @@ int PipeConnection::open() {
 
   switch(opmode) {
   case 0:
-    strcpy(z,_("Engine running"));
+    g_strlcpy(z,_("Engine running"),256);
   case 1:
-    sprintf(z,_("Connected to %s (%s)"),HostName,HostAddress);
+    snprintf(z,256,_("Connected to %s (%s)"),HostName,HostAddress);
     break;
   }
   if (!Quiet)
@@ -782,6 +874,8 @@ int PipeConnection::open() {
 }
 
 void PipeConnection::close() {
+  global.debug("PipeConnection","close");
+  cleanUp();
   ::close(pin);
   ::close(pout);
   netring.remove(this);
@@ -795,7 +889,7 @@ int PipeConnection::readLine(char *tbuffer,int limit) {
   return(Connected?innerReadLine(tbuffer,limit,pin):-1);
 }
 
-void PipeConnection::writeLine(char *obuffer) {
+void PipeConnection::writeLine(const char *obuffer) {
   int ew,rem,ec,tol=0;
   fd_set wfd;
   struct timeval tv;
@@ -814,17 +908,21 @@ void PipeConnection::writeLine(char *obuffer) {
   //  cerr << "writing [" << obuffer << "]\n";
   
   if (Connected) {
-    rem=strlen(obuffer);
+    const char *tmp;
+    tmp = global.filter(obuffer);
+
+    rem=strlen(tmp);
     for(ew=0;rem;) {
-      ec=write(pout,obuffer+ew,rem);
+      ec=write(pout,tmp+ew,rem);
       if (ec<0) { usleep(50000); ++tol; if (tol==4) break; } else { ew+=ec; rem-=ec; }
     }
     write(pout,"\n",1);
     if (global.CommLog) {
       char z[4096];
-      sprintf(z,"WROTE: %s",obuffer);
+      snprintf(z,4096,"WROTE: %s",tmp);
       global.LogAppend(z);
     }
+    if (tmp!=obuffer) free((void *)tmp);
   }
 }
 
@@ -833,10 +931,12 @@ char * PipeConnection::getError() {
 }
 
 void PipeConnection::farewellPid(int dpid) {
-  consume(pin,1536);
-  netring.remove(this);
-  // wait for data on the pipe before shutting down
-  toid=gtk_timeout_add(2000,sched_close,(gpointer)this);
+  // queue read+close, cannot be done directly here (called from a signal handler)
+  // or we get deadlocks related to malloc/free
+  if (pw_count < PW_MAX) {
+    pw_array[pw_count] = this;
+    pw_count++;
+  }
 }
 
 int PipeConnection::getReadHandle() {
@@ -893,7 +993,7 @@ int  FallBackConnection::open() {
       TimeGuard=(*current)->TimeGuard;
       return 0;
     }
-    strcpy(errorMessage,(*current)->getError());
+    g_strlcpy(errorMessage,(*current)->getError(),128);
   }
   return -1;  
 }
@@ -912,7 +1012,7 @@ int  FallBackConnection::readLine(char *tbuffer,int limit) {
     return -1;
 }
 
-void FallBackConnection::writeLine(char *tbuffer) {
+void FallBackConnection::writeLine(const char *tbuffer) {
   if (Connected)
     (*current)->writeLine(tbuffer);
 }
@@ -924,11 +1024,11 @@ int FallBackConnection::readPartial(char *tbuffer,int limit) {
     return -1;
 }
 
-int FallBackConnection::bufferMatch(char *match) {
+bool FallBackConnection::bufferMatch(const char *match) {
   if (Connected)
     return( (*current)->bufferMatch(match) );
   else
-    return 0;
+    return false;
 }
 
 char * FallBackConnection::getError() {
