@@ -1,11 +1,10 @@
-/* $Id: p2p.cc,v 1.3 2003/07/14 22:10:21 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -86,7 +85,7 @@ P2PDialog::P2PDialog() : NonModalDialog(_("eboard Direct Connection Manager")) {
   gtk_box_pack_start(GTK_BOX(vconn), cl[0], FALSE, TRUE, 2);
   gtk_box_pack_start(GTK_BOX(cl[0]), cl[1], TRUE, TRUE, 0);
   
-  Gtk::show(cl[0], cl[1], chost, cport, ct, 0);
+  Gtk::show(cl[0], cl[1], chost, cport, ct, NULL);
   
   gtk_signal_connect(GTK_OBJECT(cl[1]), "clicked",
 		     GTK_SIGNAL_FUNC(p2p_connect), (gpointer) this);
@@ -117,7 +116,7 @@ P2PDialog::P2PDialog() : NonModalDialog(_("eboard Direct Connection Manager")) {
   gtk_box_pack_start(GTK_BOX(wl[1]), wl[2], TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(wl[1]), wl[3], TRUE, TRUE, 0);
   
-  Gtk::show(wl[0],wl[1],wl[2],wl[3],wport,0);
+  Gtk::show(wl[0],wl[1],wl[2],wl[3],wport,NULL);
 
   gtk_signal_connect(GTK_OBJECT(wl[2]), "clicked",
 		     GTK_SIGNAL_FUNC(p2p_wait), (gpointer) this);
@@ -139,7 +138,7 @@ P2PDialog::P2PDialog() : NonModalDialog(_("eboard Direct Connection Manager")) {
 
   gtk_box_pack_start(GTK_BOX(vops), ot, FALSE, TRUE, 2);
 
-  Gtk::show(oname,ot,0);
+  Gtk::show(oname,ot,NULL);
 
   /* bind them */
 
@@ -167,14 +166,14 @@ P2PDialog::P2PDialog() : NonModalDialog(_("eboard Direct Connection Manager")) {
   nb->setTabPosition(GTK_POS_TOP);
 
   focused_widget = chost;
-  Gtk::show(close,bhb,vconn,vlisten,vops,v,0);
+  Gtk::show(close,bhb,vconn,vlisten,vops,v,NULL);
   setDismiss(GTK_OBJECT(close),"clicked");
 }
 
 P2PDialog::~P2PDialog() {
   int i;
   if (nb) delete nb;
-  for(i=0;i<5;i++)
+  for(i=0;i<4;i++)
     delete(bl[i]);
   if (wconn) {
     wconn->close();
@@ -191,7 +190,7 @@ void p2p_connect(GtkWidget *w, gpointer data) {
   char host[128];
   P2PProtocol *proto;
 
-  strcpy(host, gtk_entry_get_text(GTK_ENTRY(me->chost)));
+  g_strlcpy(host, gtk_entry_get_text(GTK_ENTRY(me->chost)),128);
   port = atoi(gtk_entry_get_text(GTK_ENTRY(me->cport)));
 
   if (global.network)
@@ -201,7 +200,7 @@ void p2p_connect(GtkWidget *w, gpointer data) {
     }
 
   memset(global.P2PName, 0, 64);
-  strncpy(global.P2PName, gtk_entry_get_text(GTK_ENTRY(me->oname)), 63);
+  g_strlcpy(global.P2PName, gtk_entry_get_text(GTK_ENTRY(me->oname)), 64);
   global.writeRC();
 
   global.chandler->openServer(host, port, proto = new P2PProtocol(), 0);
@@ -216,7 +215,7 @@ void p2p_wait(GtkWidget *w, gpointer data) {
   P2PDialog *me = (P2PDialog *) data;
 
   memset(global.P2PName, 0, 64);
-  strncpy(global.P2PName, gtk_entry_get_text(GTK_ENTRY(me->oname)), 63);
+  g_strlcpy(global.P2PName, gtk_entry_get_text(GTK_ENTRY(me->oname)), 64);
   global.writeRC();
 
   me->waitConnection();
@@ -241,7 +240,7 @@ void P2PDialog::waitConnection() {
     gtk_widget_set_sensitive(wbw, FALSE);
     gtk_widget_set_sensitive(wbc, TRUE);
     toid = gtk_timeout_add(500, p2p_check_incoming, (gpointer) this);
-    sprintf(z,_("Waiting for connection on port %d."),port);
+    snprintf(z,128,_("Waiting for connection on port %d."),port);
     global.status->setText(z,500);
   }
 }
@@ -259,7 +258,7 @@ void P2PDialog::cancelWait() {
   }
   gtk_widget_set_sensitive(wbw, TRUE);
   gtk_widget_set_sensitive(wbc, FALSE);
-  strcpy(z,_("Cancelled connection wait."));
+  g_strlcpy(z,_("Cancelled connection wait."),128);
   global.status->setText(z,5);
 }
 
