@@ -1,11 +1,10 @@
-/* $Id: pieces.cc,v 1.34 2003/01/19 19:24:03 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,7 +35,7 @@
 #include "tstring.h"
 #include "eboard.h"
 
-#include "fallback.xpm"
+#include "xpm/fallback.xpm"
 
 PReq::PReq(piece a, rgbptr b, int c, int d, int e, bool f) {
   p=a; dest=b; x=c; y=d; dwidth=e; onlytop=f;
@@ -45,12 +44,6 @@ PReq::PReq(piece a, rgbptr b, int c, int d, int e, bool f) {
 int PReq::operator< (PReq * a) { return(y < a->y); }
 int PReq::operator> (PReq * a) { return(y > a->y); }
 
-//
-// Hint about gdk_imlib_changed_image() :
-//
-// http://www.mail-archive.com/dia-list@lysator.liu.se/msg02435.html
-//
-
 PieceSet::PieceSet(string &name, string &sqname) {
   EboardFileFinder eff;
   string wherever;
@@ -70,15 +63,13 @@ PieceSet::PieceSet(string &name, string &sqname) {
   memset(cpath,0,512);
   wherever.copy(cpath,512);
 
-  isrc=gdk_imlib_load_image(cpath);
-  if (isrc==0) {
+  isrc = new CImg(cpath);  
+  if (isrc==0 || !(isrc->ok)) {
     cerr << "<PieceSet::PieceSet>";
     cerr << _(" ** PNG LOAD FAILED: using internal low-res pieceset. It'll look ugly.\n");
     isrc=FallbackLoad();
   }
 
-  gdk_imlib_changed_image(isrc);
-
   // use squares from other file ?
   if (Name.compare(SqName)) {
     if (!eff.find(SqName, wherever))
@@ -87,25 +78,23 @@ PieceSet::PieceSet(string &name, string &sqname) {
       loadSquares(wherever);
   }
 
-  side = height = isrc->rgb_width / 7;
+  side = height = isrc->W / 7;
   bottom_only=false;
   top_only=false;
   queuef=false;
 
-  i = (isrc->rgb_width  * 10) / isrc->rgb_height;
+  i = (isrc->W  * 10) / isrc->H;
 
   if ((i<34)||(i>36)) {
     extruded=true;
-    height=isrc->rgb_height / 2;
+    height=isrc->H / 2;
   } else {
     extruded=false;
   }
 
   initCache();
 
-  bgcolor =((int)(isrc->rgb_data[0]))<<16;
-  bgcolor|=((int)(isrc->rgb_data[1]))<<8;
-  bgcolor|=((int)(isrc->rgb_data[2]));
+  bgcolor= isrc->get(0,0);
   alphamask=0;
   calcAlphaMask();
 }
@@ -114,8 +103,7 @@ PieceSet::PieceSet(PieceSet *src) {
   Name=src->Name;
   SqName=src->SqName;
   initCache();
-  isrc=gdk_imlib_clone_image(src->isrc);
-  gdk_imlib_changed_image(isrc);
+  isrc=new CImg(src->isrc);
   side=src->side;
   height=src->height;
   extruded=src->extruded;
@@ -131,7 +119,7 @@ PieceSet::~PieceSet() {
   list<PReq *>::iterator qi;
   if (alphamask)
     g_free(alphamask);
-  gdk_imlib_destroy_image(isrc);
+  delete(isrc);
   clearCache();
 
   if (!myq.empty()) {
@@ -150,7 +138,7 @@ string & PieceSet::getSquareName() {
 }
 
 void PieceSet::loadSquares(string &path) {
-  GdkImlibImage *tmp,*aux;
+  CImg *tmp,*aux;
   char cpath[512];
 
   int w1,w2,h1,h2;
@@ -161,60 +149,57 @@ void PieceSet::loadSquares(string &path) {
 
   global.debug("PieceSet","loadSquares",cpath);
 
-  tmp=gdk_imlib_load_image(cpath);
+  tmp = new CImg(cpath);
 
-  if (tmp==0) {
+  if (tmp==0 || !(tmp->ok)) {
     cerr << "<PieceSet::loadSquares>";
     cerr << _(" ** PNG LOAD FAILED: using internal low-res pieceset. It'll look ugly.\n");
     tmp=FallbackLoad();
   }
 
-  gdk_imlib_changed_image(tmp);
+  w1=isrc->W;
+  w2=tmp->W;
 
-  w1=isrc->rgb_width;
-  w2=tmp->rgb_width;
-
-  h1=isrc->rgb_height;
-  h2=tmp->rgb_height;
+  h1=isrc->H;
+  h2=tmp->H;
 
   w=w2/7;
 
-  bitblt(tmp->rgb_data, tmp->rgb_data,
+  bitblt(tmp->data, tmp->data,
 	 6*w, 0, w2,
 	 0, 0, w2,
 	 w, w);
 
-  bitblt(tmp->rgb_data, tmp->rgb_data,
+  bitblt(tmp->data, tmp->data,
 	 6*w, h2/2, w2,
 	 0, w, w2,
 	 w, w);
 
-  gdk_imlib_crop_image(tmp,0,0,w, 2*w);
+  tmp->crop(0,0,w,2*w);
 
   if ( (w1!=w2) ) {
     w=w1/7;
-    aux=gdk_imlib_clone_scaled_image(tmp,w,2*w);
-    gdk_imlib_destroy_image(tmp);
-    tmp=aux;
-    gdk_imlib_changed_image(tmp);
+    aux = tmp->scale(w,2*w);
+    delete(tmp);
+    tmp = aux;
   }
 
   x=(w1/7)*6;
   w=(w1/7);
 
   // white square
-  bitblt(tmp->rgb_data,isrc->rgb_data,
+  bitblt(tmp->data,isrc->data,
 	 0,0,w,
 	 x,0,w1, 
 	 w,w);
 
   // black square
-  bitblt(tmp->rgb_data,isrc->rgb_data,
+  bitblt(tmp->data,isrc->data,
 	 0, w, w,
-	 x, isrc->rgb_height / 2, w1, 
+	 x, isrc->H / 2, w1, 
 	 w,w);
 
-  gdk_imlib_destroy_image(tmp);
+  delete(tmp);
 }
 
 void PieceSet::initCache() {
@@ -284,7 +269,7 @@ void PieceSet::clearCache() {
 }
 
 void PieceSet::scale(int newside) {
-  GdkImlibImage *ni;
+  CImg *ni;
   float factor;
   int newheight;
 
@@ -297,12 +282,13 @@ void PieceSet::scale(int newside) {
   else
     newheight = newside;
 
-  ni=gimp_scale_preserve(isrc,alphamask,newside*7,newheight*2);
+  ni = isrc->alphaScale(alphamask,newside*7,newheight*2,extruded);
+  bgcolor = 0x808080;
 
   side=newside;
   height=newheight;
 
-  gdk_imlib_destroy_image(isrc);
+  delete(isrc);
 
   isrc=ni;
   calcAlphaMask();
@@ -310,357 +296,6 @@ void PieceSet::scale(int newside) {
   buildCache();
 }
 
-#define EPSILON 0.0001
-
-GdkImlibImage *
-PieceSet::gimp_scale_preserve(GdkImlibImage *src, rgbptr alpha, int nw,int nh) {
-
-  GdkImlibImage *img, *a, *b;
-  rgbptr s,d;
-  hati   p, R,G,B;
-  int i, q, z;
-  int ew,eh,missing;
-
-  a=gdk_imlib_clone_scaled_image(src,src->rgb_width,src->rgb_height);
-  
-  // make a fake rgb alpha mask
-  s=alpha;
-  d=a->rgb_data;  
-
-  i=src->rgb_width * src->rgb_height;
-  for(;i;i--) {
-    p= *(s++) << 7;
-    *(d++) = p;
-    *(d++) = p;
-    *(d++) = p;
-  }
-
-  R=G=B=0x80;
-  bgcolor=pixel(R,G,B); 
-
-  // substitute bgcolor
-  i=src->rgb_width * src->rgb_height;
-  d=src->rgb_data;
-  z=pixel(*d,*(d+1),*(d+2));
-  for(;i;i--) {
-      q=pixel(*d,*(d+1),*(d+2));
-      if (q==z) *d=*(d+1)=*(d+2)=R; 
-      d+=3;
-  }
-
-  img=gimp_scale(src,nw,nh);
-  b=gimp_scale(a, nw, nh);
-  gdk_imlib_destroy_image(a);
-  // gdk_imlib_save_image_to_ppm(b, "debug2.ppm");
-
-  // fix the last column before the squares
-  d=img->rgb_data + 3* ((nw/7)*6 - 1);
-  for(i=0;i<nh;i++) {
-    *d     = R;
-    *(d+1) = G;
-    *(d+2) = B;
-    d+=3*nw;
-  }
-  
-  s=b->rgb_data;
-  d=img->rgb_data;
-  i=nw * nh * 3;
-
-  for(;i;i-=3) {
-    if ( (*s) > 0x60 ) {
-      *(d) = R;
-      *(d+1) = G;
-      *(d+2) = B;
-    }
-    s+=3;
-    d+=3;
-  }
-
-  // fix annoying border (d)effect on extruded sets
-  if (extruded) {
-    ew=nw/7;
-    eh=nh/2;
-
-    i=ew-1;
-    missing=0;
-
-    d=img->rgb_data+ 3* (nw*i + 6*ew + 2);
-    while(i>0) {
-      q=pixel(*d,*(d+1),*(d+2));
-
-      if (q!=bgcolor)
-	break;
-
-      ++missing;
-      --i;
-      d-=3*nw;
-    }
-
-    if (missing < ew/2) {
-      bitblt( img->rgb_data, img->rgb_data, // !!!
-	      6*ew, ew-(2*missing), nw,       // src
-	      6*ew, ew-missing    , nw,       // dest
-	      ew, missing );
-      
-      bitblt( img->rgb_data, img->rgb_data, // !!!
-	      6*ew, eh+ew-(2*missing), nw,     //src
-	      6*ew, eh+ew-missing, nw,       //dest
-	      ew, missing );
-    }
-
-  }
-
-  gdk_imlib_destroy_image(b);
-  gdk_imlib_changed_image(img);
-  //  gdk_imlib_save_image_to_ppm(img, "debug.ppm");
-
-  return(img);
-}
-
-// taken from gimp 1.0.4, scale_region(...) from app/paint_funcs.c
-GdkImlibImage * PieceSet::gimp_scale(GdkImlibImage *gsrc,int nw,int nh) {
-  
-
-  unsigned char * src_m1, * src, * src_p1, * src_p2;
-  unsigned char * s_m1, * s, * s_p1, * s_p2;
-  unsigned char * dest, * d;
-  double * row, * r;
-  int src_row, src_col;
-  int bytes, b;
-  int width, height;
-  int orig_width, orig_height;
-  double x_rat, y_rat;
-  double x_cum, y_cum;
-  double x_last, y_last;
-  double * x_frac, y_frac, tot_frac;
-  float dx, dy;
-  int i, j;
-  int frac;
-  int advance_dest_x, advance_dest_y;
-  int minus_x, plus_x, plus2_x;
-  int scale_type;
-
-  GdkImlibImage *img;
-  int stride;
-
-  orig_width = gsrc->rgb_width;
-  orig_height = gsrc->rgb_height;
-
-  width = nw;
-  height = nh;
-
-  img=gdk_imlib_clone_scaled_image(gsrc, nw, nh);  
-
-  /*  Some calculations...  */
-  bytes = 3;
-  stride= orig_width*bytes;
-
-  /*  the data pointers...  */
-  src_m1 = (unsigned char *) g_malloc (orig_width * bytes);
-  src    = (unsigned char *) g_malloc (orig_width * bytes);
-  src_p1 = (unsigned char *) g_malloc (orig_width * bytes);
-  src_p2 = (unsigned char *) g_malloc (orig_width * bytes);
-  dest   = (unsigned char *) g_malloc (width * bytes);
-
-  /*  find the ratios of old x to new x and old y to new y  */
-  x_rat = (double) orig_width / (double) width;
-  y_rat = (double) orig_height / (double) height;
-
-  /*  determine the scale type  */
-  if (x_rat < 1.0 && y_rat < 1.0)
-    scale_type = 0; // MagnifyX_MagnifyY
-  else if (x_rat < 1.0 && y_rat >= 1.0)
-    scale_type = 1; // MagnifyX_MinifyY
-  else if (x_rat >= 1.0 && y_rat < 1.0)
-    scale_type = 2; // MinifyX_MagnifyY
-  else
-    scale_type = 3; // MinifyX_MinifyY
-
-  /*  allocate an array to help with the calculations  */
-  row    = (double *) g_malloc (sizeof (double) * width * bytes);
-  x_frac = (double *) g_malloc (sizeof (double) * (width + orig_width));
-
-  /*  initialize the pre-calculated pixel fraction array  */
-  src_col = 0;
-  x_cum = (double) src_col;
-  x_last = x_cum;
-
-  for (i = 0; i < width + orig_width; i++)
-    {
-      if (x_cum + x_rat <= (src_col + 1 + EPSILON))
-	{
-	  x_cum += x_rat;
-	  x_frac[i] = x_cum - x_last;
-	}
-      else
-	{
-	  src_col ++;
-	  x_frac[i] = src_col - x_last;
-	}
-      x_last += x_frac[i];
-    }
-  
-  /*  clear the "row" array  */
-  memset (row, 0, sizeof (double) * width * bytes);
-  
-  /*  counters...  */
-  src_row = 0;
-  y_cum = (double) src_row;
-  y_last = y_cum;
-
-  /*  Get the first src row  */
-  memcpy(src, gsrc->rgb_data + src_row * stride, stride);
-
-  /*  Get the next two if possible  */
-  if (src_row < (orig_height - 1))
-    memcpy(src_p1, gsrc->rgb_data + (src_row + 1) * stride, stride);
-
-  if ((src_row + 1) < (orig_height - 1))
-    memcpy(src_p2, gsrc->rgb_data + (src_row + 2) * stride, stride);
-
-  /*  Scale the selected region  */
-  i = height;
-  while (i)
-    {
-      src_col = 0;
-      x_cum = (double) src_col;
-
-      /* determine the fraction of the src pixel we are using for y */
-      if (y_cum + y_rat <= (src_row + 1 + EPSILON))
-	{
-	  y_cum += y_rat;
-	  dy = y_cum - src_row;
-	  y_frac = y_cum - y_last;
-	  advance_dest_y = TRUE;
-	}
-      else
-	{
-	  y_frac = (src_row + 1) - y_last;
-	  dy = 1.0;
-	  advance_dest_y = FALSE;
-	}
-
-      y_last += y_frac;
-
-      s = src;
-      s_m1 = (src_row > 0) ? src_m1 : src;
-      s_p1 = (src_row < (orig_height - 1)) ? src_p1 : src;
-      s_p2 = ((src_row + 1) < (orig_height - 1)) ? src_p2 : s_p1;
-
-      r = row;
-
-      frac = 0;
-      j = width;
-
-      while (j)
-	{
-	  if (x_cum + x_rat <= (src_col + 1 + EPSILON))
-	    {
-	      x_cum += x_rat;
-	      dx = x_cum - src_col;
-	      advance_dest_x = TRUE;
-	    }
-	  else
-	    {
-	      dx = 1.0;
-	      advance_dest_x = FALSE;
-	    }
-	  
-	  tot_frac = x_frac[frac++] * y_frac;
-
-	  minus_x = (src_col > 0) ? -bytes : 0;
-	  plus_x = (src_col < (orig_width - 1)) ? bytes : 0;
-	  plus2_x = ((src_col + 1) < (orig_width - 1)) ? bytes * 2 : plus_x;
-
-	  switch (scale_type)
-	    {
-	    case 0: // MagnifyX_MagnifyY
-	      for (b = 0; b < bytes; b++)
-		r[b] += ((1 - dy) * ((1 - dx) * s[b] + dx * s[b+plus_x]) +
-			 dy  * ((1 - dx) * s_p1[b] + dx * s_p1[b+plus_x])) * tot_frac;
-	      break;
-	    case 1: // MagnifyX_MinifyY
-	      for (b = 0; b < bytes; b++)
-		r[b] += (s[b] * (1 - dx) + s[b+plus_x] * dx) * tot_frac;
-	      break;
-	    case 2: // MinifyX_MagnifyY
-	      for (b = 0; b < bytes; b++)
-		r[b] += (s[b] * (1 - dy) + s_p1[b] * dy) * tot_frac;
-	      break;
-	    case 3: //MinifyX_MinifyY
-	      for (b = 0; b < bytes; b++)
-		r[b] += s[b] * tot_frac;
-	      break;
-	    }
-
-	  if (advance_dest_x)
-	    {
-	      r += bytes;
-	      j--;
-	    }
-	  else
-	    {
-	      s_m1 += bytes;
-	      s    += bytes;
-	      s_p1 += bytes;
-	      s_p2 += bytes;
-	      src_col++;
-	    }
-	}
-
-      if (advance_dest_y)
-	{
-	  tot_frac = 1.0 / (x_rat * y_rat);
-
-	  /*  copy "row" to "dest"  */
-	  d = dest;
-	  r = row;
-
-	  j = width;
-	  while (j--)
-	    {
-	      b = bytes;
-	      while (b--)
-		*d++ = (unsigned char) (*r++ * tot_frac);
-	    }
-
-	  /*  set the pixel region span  */
-	  memcpy(img->rgb_data + (width * 3) * (height-i), dest, width * 3);
-
-	  /*  clear the "row" array  */
-	  memset (row, 0, sizeof (double) * width * bytes);
-
-	  i--;
-	}
-      else
-	{
-	  /*  Shuffle pointers  */
-	  s = src_m1;
-	  src_m1 = src;
-	  src = src_p1;
-	  src_p1 = src_p2;
-	  src_p2 = s;
-
-	  src_row++;
-	  if ((src_row + 1) < (orig_height - 1))
-	    memcpy(src_p2, gsrc->rgb_data + (src_row + 2) * stride, stride);
-	}
-    }
-
-  /*  free up temporary arrays  */
-  g_free (row);
-  g_free (x_frac);
-  g_free (src_m1);
-  g_free (src);
-  g_free (src_p1);
-  g_free (src_p2);
-  g_free (dest);
-
-  gdk_imlib_changed_image(img);
-  return img;
-}
-
-
 void PieceSet::calcAlphaMask() {
   int c;
   unsigned char r,g,b;
@@ -673,7 +308,7 @@ void PieceSet::calcAlphaMask() {
   alphamask=(rgbptr)g_malloc(c);
   memset(alphamask,0,c);
   
-  t=isrc->rgb_data;
+  t=isrc->data;
   am=alphamask;
 
   r=bgcolor>>16;
@@ -717,7 +352,7 @@ void PieceSet::drawPiece(piece p, GdkPixmap *dest, GdkGC *gc,
 
   for(i=0;i<height;i++) {
 
-    r=isrc->rgb_data+3*(7*side*(i+sy)+sx); // piece
+    r=isrc->data+3*(7*side*(i+sy)+sx); // piece
     S=0;
 
     for(j=0;j<side;j++) {
@@ -805,7 +440,7 @@ void PieceSet::drawOutlinedPiece(piece p,GdkPixmap *dest,GdkGC *gc,
 
   for(i=0;i<height;i++) {
 
-    r=isrc->rgb_data+3*(7*side*(i+sy)+sx); // piece
+    r=isrc->data+3*(7*side*(i+sy)+sx); // piece
     S=0;
 
     for(j=0;j<side;j++) {
@@ -898,12 +533,12 @@ void PieceSet::drawPiece(piece p,rgbptr dest,int x,int y,int dwidth) {
 
     if (top_only) hlt=height-side;
     if (sc<height)
-      bitblt_a(isrc->rgb_data,dest,
+      bitblt_a(isrc->data,dest,
 	       sx,sy+sc,7*side,
 	       x,top,dwidth,
-	       side,height-sc-hlt);
+	       side,height-sc-hlt,alphamask);
   } else
-    bitblt_a(isrc->rgb_data,dest,sx,sy,7*side,x,y,dwidth,side,side);    
+    bitblt_a(isrc->data,dest,sx,sy,7*side,x,y,dwidth,side,side,alphamask);    
 }
 
 
@@ -928,7 +563,7 @@ void PieceSet::drawSquare(int dark,rgbptr dest,int x,int y,int dwidth) {
   } else {
     if (dark) sy=height;
     sx=6*side;  
-    bitblt(isrc->rgb_data,dest,sx,sy,7*side,x,y,dwidth,side,side);
+    bitblt(isrc->data,dest,sx,sy,7*side,x,y,dwidth,side,side);
   }
   rect_outline(dest,x,y,side+1,side+1,dwidth);
 }
@@ -959,56 +594,6 @@ void PieceSet::drawPieceAndSquare(piece p,rgbptr dest,
     queueRequest(new PReq(p,dest,x,y,dwidth,true));
 }
 
-void PieceSet::bitblt(rgbptr src,rgbptr dest,int sx,int sy,int sw,
-	    int dx,int dy,int dw,int w,int h) {
-  register rgbptr d, s;
-  register int wcd;
-  static int slg,dlg, wcd0;
-  d=dest+3*(dw*dy+dx);
-  s=src+3*(sw*sy+sx);
-  dlg=3*(dw-w);
-  slg=3*(sw-w);
-  wcd0=3*w;
-  while(h) {
-    wcd=wcd0;
-    while(wcd--)
-      *(d++)=*(s++);
-    d+=dlg;
-    s+=slg;
-    --h;
-  }
-}
-
-void PieceSet::bitblt_a(rgbptr src,rgbptr dest,int sx,int sy,int sw,
-			int dx,int dy,int dw,int w,int h) {
-  register rgbptr d, s, a;
-  static int wcd,slg,dlg,alg;
-  s=src+3*(sw*sy+sx);
-  d=dest+3*(dw*dy+dx);
-  a=alphamask+(sw*sy+sx);
-  slg=3*(sw-w);
-  dlg=3*(dw-w);
-  alg=sw-w;
-  while(h) {
-    wcd=w;
-    while(wcd--) {
-      if (*a) {
-	d+=3;
-	s+=3;
-      } else {
-	*(d++)=*(s++);
-	*(d++)=*(s++);
-	*(d++)=*(s++);
-      }
-      ++a;
-    }
-    d+=dlg;
-    s+=slg;
-    a+=alg;
-    --h;
-  }
-}
-
 void PieceSet::rect_outline(rgbptr img,int x,int y,int w,int h,int iw) {
   int i;
   memset(img+3*(iw*y+x),0,3*w);
@@ -1021,13 +606,13 @@ void PieceSet::rect_outline(rgbptr img,int x,int y,int w,int h,int iw) {
 
 int  PieceSet::getPixel(int x,int y) {
   rgbptr src;
-  src=isrc->rgb_data+(y*(isrc->rgb_width)+x)*3;
+  src=isrc->data+(y*(isrc->W)+x)*3;
   return(pixel(src[0],src[1],src[2]));
 }
 
 void PieceSet::setPixel(int x,int y,int v) {
   rgbptr src;
-  src=isrc->rgb_data+(y*(isrc->rgb_width)+x)*3;
+  src=isrc->data+(y*(isrc->W)+x)*3;
   src[0]=(unsigned char)pixr(v);
   src[1]=(unsigned char)pixg(v);
   src[2]=(unsigned char)pixb(v);
@@ -1035,7 +620,7 @@ void PieceSet::setPixel(int x,int y,int v) {
 
 void PieceSet::xSetPixel(rgbptr img,int x,int y,int v) {
   rgbptr src;
-  src=img+(y*(isrc->rgb_width)+x)*3;
+  src=img+(y*(isrc->W)+x)*3;
   src[0]=(unsigned char)pixr(v);
   src[1]=(unsigned char)pixg(v);
   src[2]=(unsigned char)pixb(v);
@@ -1044,7 +629,7 @@ void PieceSet::xSetPixel(rgbptr img,int x,int y,int v) {
 int PieceSet::alpha(int x,int y) {
   rgbptr a;
   if (!alphamask) return 0;
-  a=alphamask+(y*(isrc->rgb_width)+x);
+  a=alphamask+(y*(isrc->W)+x);
   return( (int)(a[0]) );
 }
 
@@ -1078,11 +663,11 @@ void PieceSet::endQueueing() {
 }
 
 
-GdkImlibImage * PieceSet::FallbackLoad() {
+CImg * PieceSet::FallbackLoad() {
   tstring t;
   int w,h,c,i,j;
   rgbptr data, p;
-  GdkImlibImage *img;
+  CImg *img;
   string *s;
 
   int cmap[256];
@@ -1116,7 +701,8 @@ GdkImlibImage * PieceSet::FallbackLoad() {
       *(p++)=pixb(cmap[fallback_xpm[1+c+j][i]]);
     }
 
-  img=gdk_imlib_create_image_from_data(data,0,w,h);
+  img = new CImg(w,h);
+  memcpy(img->data, data, h*(img->rowlen));
   g_free(data);
   return img;
 }
@@ -1290,75 +876,3 @@ void VecDetail::draw(GdkPixmap *dest, GdkGC *gc,int x,int y,int sz1,int sz2) {
 		  x+(x2[i]*sz1)/sz2,y+(y2[i]*sz1)/sz2);
 }
 
-// ----------- RGB <-> YCbCr transform
-
-int ColorSpace::lighter(int triplet) {
-  int a,b;
-  a=RGB2YCbCr(triplet);
-  b=pixr(a);
-  b=(256+b)/2;
-  b=YCbCr2RGB(pixel(b,pixg(a),pixb(a)));
-  return b;
-}
-
-int ColorSpace::darker(int triplet) {
-  int a,b;
-  a=RGB2YCbCr(triplet);
-  b=pixr(a);
-  b/=2;
-  b=YCbCr2RGB(pixel(b,pixg(a),pixb(a)));
-  return b;
-}
-
-int ColorSpace::RGB2YCbCr(int triplet) {
-  float y,cb,cr;
-
-  y=0.257*(float)pixr(triplet)+
-    0.504*(float)pixg(triplet)+
-    0.098*(float)pixb(triplet)+16.0;
-
-  cb=-0.148*(float)pixr(triplet)-
-      0.291*(float)pixg(triplet)+
-      0.439*(float)pixb(triplet)+
-      128.0;
-
-  cr=0.439*(float)pixr(triplet)-
-     0.368*(float)pixg(triplet)-
-     0.071*(float)pixb(triplet)+
-     128.0;
-
-  return(pixel((int)y,(int)cb,(int)cr));
-}
-
-int ColorSpace::YCbCr2RGB(int triplet) {
-  float r,g,b;
-
-  r=1.164*((float)pixr(triplet)-16.0)+
-    1.596*((float)pixb(triplet)-128.0);
-
-  g=1.164*((float)pixr(triplet)-16.0)-
-    0.813*((float)pixb(triplet)-128.0)-
-    0.392*((float)pixg(triplet)-128.0);
-  
-  b=1.164*((float)pixr(triplet)-16.0)+
-    2.017*((float)pixg(triplet)-128.0);
-
-  return(pixel((int)r,(int)g,(int)b));
-}
-
-int ColorSpace::pixr(int triplet) {
-  return((triplet>>16)&0xff);
-}
-
-int ColorSpace::pixg(int triplet) {
-  return((triplet>>8)&0xff);
-}
-
-int ColorSpace::pixb(int triplet) {
-  return(triplet&0xff);
-}
-
-int ColorSpace::pixel(int r,int g,int b) {
-  int v=((r&0xff)<<16)|((g&0xff)<<8)|(b&0xff);
-  return(v);
-}
