@@ -1,11 +1,10 @@
-/* $Id: position.cc,v 1.61 2008/02/18 13:21:05 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -248,7 +247,7 @@ void Position::invalidate() {
       square[i][j]++;
 }
 
-void Position::setLastMove(char *s) {
+void Position::setLastMove(const char *s) {
   LastMove=s;
 }
 
@@ -260,7 +259,7 @@ string & Position::getLastMove() {
   return(LastMove);
 }
 
-void Position::moveAnyNotation(char *m,piece color,variant Vr) {
+void Position::moveAnyNotation(const char *m,piece color,variant Vr) {
   int i,x,y,ml;
   piece pr;
   char xlate[12];
@@ -290,6 +289,8 @@ void Position::moveAnyNotation(char *m,piece color,variant Vr) {
   }
   
   ml = strlen(m);
+  while(ml > 0 && (m[ml-1] == '!' || m[ml-1] == '?'))
+    ml--;
 
   if ( isupper(m[0]) || m[1]=='x' || m[2]=='x' || m[0]=='o' || ml==2 || 
        ml == 3 || m[2]=='=' || ( ml >= 4 && ( m[3]=='+' || m[3]=='#' ) )
@@ -320,11 +321,11 @@ void Position::moveAnyNotation(char *m,piece color,variant Vr) {
     }
 }
 
-void Position::moveStdNotation(char *m,piece color,variant Vr) {
+void Position::moveStdNotation(const char *m,piece color,variant Vr) {
   int from[2],to[2];
   int istake, isdrop;
   piece what, prom;
-  char *p;
+  const char *p;
   int cy;
   global.debug("Position","moveStdNotation",m);
 
