@@ -1,11 +1,10 @@
-/* $Id: proto_xboard.cc,v 1.36 2003/07/02 23:50:24 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -52,15 +51,18 @@ XBoardProtocol::XBoardProtocol() {
   Variant=REGULAR;
   want_path_pane=1;
   supports_setboard=false;
+  requires_usermove=false;
   timeout_id=-1;
   need_handshake=true; // send xboard/protover 2
+  Finalized = false;
+  InitDone  = false;
 
   got_init_pos = false;
   just_sent_fen = false;
 
   timecontrol.setSecondsPerMove(20);
 
-  strcpy(ComputerName,_("Computer"));
+  g_strlcpy(ComputerName,_("Computer"),64);
   strcpy(EngineCommandLine,"gnuchess");
   strcpy(EngineRunDir,"~/.eboard/eng-out");
 
@@ -137,6 +139,18 @@ void XBoardProtocol::receiveString(char *netstring) {
 
   if (IllegalMove.match(netstring)) {
 
+    if (!InitDone) {
+      // most likely engine does not support xboard protocol v2.
+      // and currently we do not support these.
+      if (timeout_id >= 0) {
+	gtk_timeout_remove(timeout_id);
+	timeout_id = -1;
+      }
+      finalize();
+      global.status->setText(_("<XBoard Protocol> Incompatible Engine Protocol")); // TRANSLATE
+      goto just_print;
+    }
+
     if (just_sent_fen)
       if ((CurrentPosition.sidehint && EngineWhite) ||
 	  (!CurrentPosition.sidehint && !EngineWhite) ) {
@@ -156,10 +170,10 @@ void XBoardProtocol::receiveString(char *netstring) {
     if (timecontrol.mode == TC_SPM) {
       if (EngineWhite) bc=0; else wc=0;
     }
-    refgame->updatePosition(CurrentPosition, MoveNumber,
-			    EngineWhite?1:0, 
-			    wc,bc,
-			    _("illegal move!"));
+    refgame->updatePosition2(CurrentPosition, MoveNumber,
+			     EngineWhite?1:0, 
+			     wc,bc,
+			     _("illegal move!"));
     global.status->setText(netstring,15);
     global.BoardList.front()->setCanMove(true);
     return;
@@ -178,16 +192,16 @@ void XBoardProtocol::receiveString(char *netstring) {
     CurrentPosition.moveAnyNotation(Moved[moved].getRToken(0),
                                     (MoveBlack?BLACK:WHITE),Variant);
     if (EngineWhite)
-      sprintf(b,"%d. %s",MoveNumber,a);
+      snprintf(b,64,"%d. %s",MoveNumber,a);
     else
-      sprintf(b,"%d. ... %s",MoveNumber,a);
+      snprintf(b,64,"%d. ... %s",MoveNumber,a);
 
     refgame=getGame();
 
     wc = bc = CLOCK_UNCHANGED;
     if (timecontrol.mode == TC_ICS) {
       if (MoveNumber == 1)
-	wc = bc = timecontrol.startValue();
+	wc = bc = 1000 * timecontrol.startValue();
       else
 	refgame->incrementActiveClock(timecontrol.value[1]);
     }    
@@ -195,10 +209,10 @@ void XBoardProtocol::receiveString(char *netstring) {
       if (EngineWhite) bc=0; else wc=0;
     }
 
-    refgame->updatePosition(CurrentPosition, MoveNumber,
-                            !MoveBlack, 
-			    wc, bc,
-			    b, true);
+    refgame->updatePosition2(CurrentPosition, MoveNumber,
+			     !MoveBlack, 
+			     wc, bc,
+			     b, true);
     advanceMove();
     global.BoardList.front()->setCanMove(true);
     global.opponentMoved();
@@ -240,8 +254,8 @@ void XBoardProtocol::backMove() {
 
 void XBoardProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
   ChessGame *refgame;
-  char move[6];
-  char mymove[12],ppfmm[18]; // [p]retty [p]rint [f]or [m]y [m]ove
+  char move[6], umove[15];
+  char mymove[12],ppfmm[32]; // [p]retty [p]rint [f]or [m]y [m]ove
   piece pp;
   int wc, bc;
 
@@ -254,14 +268,14 @@ void XBoardProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
   CurrentPosition.moveCartesian(x1,y1,x2,y2,Variant,true);
 
   if (EngineWhite)
-    sprintf(ppfmm,"%d. ... %s",MoveNumber,mymove);
+    snprintf(ppfmm,32,"%d. ... %s",MoveNumber,mymove);
   else
-    sprintf(ppfmm,"%d. %s",MoveNumber,mymove);
+    snprintf(ppfmm,32,"%d. %s",MoveNumber,mymove);
 
   wc = bc = CLOCK_UNCHANGED;
   if (timecontrol.mode == TC_ICS) {
     if (MoveNumber == 1)
-      wc = bc = timecontrol.startValue();
+      wc = bc = 1000 * timecontrol.startValue();
     else
       refgame->incrementActiveClock(timecontrol.value[1]);
   }
@@ -269,11 +283,11 @@ void XBoardProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
     if (EngineWhite) wc=0; else bc=0;
   }
 
-  refgame->updatePosition(CurrentPosition,
-                          MoveNumber,
-                          !MoveBlack,
-			  wc,bc,
-			  ppfmm);
+  refgame->updatePosition2(CurrentPosition,
+			   MoveNumber,
+			   !MoveBlack,
+			   wc,bc,
+			   ppfmm);
   move[4]=0;
   move[5]=0;
   move[0]='a'+x1;
@@ -291,20 +305,32 @@ void XBoardProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
     }
   }
 
-  global.network->writeLine(move);
+  if (requires_usermove) {
+    strcpy(umove,"usermove ");
+    strcat(umove,move);
+    global.network->writeLine(umove);
+  } else {
+    global.network->writeLine(move);
+  }
   advanceMove();
   global.BoardList.front()->setCanMove(false);
 }
 
 void XBoardProtocol::finalize() {
   ChessGame *refgame;
-  refgame=getGame();
-  if (refgame) {
-    refgame->GameNumber=global.nextFreeGameId(XBOARD_GAME+1);
-    refgame->endGame("interrupted",UNDEF);
+
+  if (!Finalized) {
+    Finalized = true;
+    refgame=getGame(true);
+    if (refgame!=NULL) {
+      refgame->GameNumber=global.nextFreeGameId(XBOARD_GAME+1);
+      refgame->endGame("interrupted",UNDEF);
+    }
+    if (global.network!=NULL) {
+      global.network->close();
+      global.network->sendReadNotify();
+    }
   }
-  if (global.network)
-    global.network->close();
 }
 
 void XBoardProtocol::gameOver(ExtPatternMatcher &pm,GameResult gr,
@@ -314,8 +340,7 @@ void XBoardProtocol::gameOver(ExtPatternMatcher &pm,GameResult gr,
   ChessGame *refgame;  
 
   if (hasreason) {
-    reason[63]=0;
-    strncpy(reason,pm.getStarToken(0),64);
+    g_strlcpy(reason,pm.getStarToken(0),64);
   } else
     strcpy(reason,"Game over"); // I can't think of anything better...
 
@@ -333,17 +358,36 @@ void XBoardProtocol::gameOver(ExtPatternMatcher &pm,GameResult gr,
   if (global.AppendPlayed) {
     if  (refgame->savePGN(global.AppendFile,true)) {
       char z[128];
-      sprintf(z,_("Game appended to %s"),global.AppendFile);
+      snprintf(z,128,_("Game appended to %s"),global.AppendFile);
       global.status->setText(z,10);
     }
   }
 
   global.network->close();
+  global.network->sendReadNotify();
 }
 
 void XBoardProtocol::resign() {
-  global.network->writeLine("resign");
+  ChessGame *refgame;
+  global.gameLost();
+  refgame=getGame();
+  if (EngineWhite) {
+    global.network->writeLine("result 0-1 {Black Resigns}");
+    refgame->endGame("Black resigned", WHITE_WIN);
+  } else {
+    global.network->writeLine("result 1-0 {White Resigns}");
+    refgame->endGame("White resigned", WHITE_WIN);
+  }
   global.status->setText(_("Resigned."),10);
+  if (global.AppendPlayed) {
+    if  (refgame->savePGN(global.AppendFile,true)) {
+      char z[128];
+      snprintf(z,128,_("Game appended to %s"),global.AppendFile);
+      global.status->setText(z,10);
+    }
+  }
+  global.network->close();
+  global.network->sendReadNotify();
 }
 
 void XBoardProtocol::draw() {
@@ -362,8 +406,8 @@ void XBoardProtocol::abort() {
 
 void XBoardProtocol::retractMove() {
   ChessGame *ref;
-  ref = getGame();
-  if (ref && MoveNumber>1) {
+  ref = getGame(true);
+  if (ref!=NULL && MoveNumber>1) {
     if ((!MoveBlack && EngineWhite)||
 	(MoveBlack && !EngineWhite)) {
       global.status->setText(_("You can only retract when it's your turn to move."),15);
@@ -375,8 +419,8 @@ void XBoardProtocol::retractMove() {
       --MoveNumber;
       memset(z,0,64);
       p.getLastMove().copy(z,63);
-      ref->updatePosition(p,MoveNumber,MoveBlack,
-			  CLOCK_UNCHANGED, CLOCK_UNCHANGED,z);
+      ref->updatePosition2(p,MoveNumber,MoveBlack,
+			   CLOCK_UNCHANGED, CLOCK_UNCHANGED,z);
       global.network->writeLine("remove");
       global.status->setText(_("Retracted last move."),15);
       CurrentPosition = p;
@@ -384,10 +428,10 @@ void XBoardProtocol::retractMove() {
   }
 }
 
-ChessGame * XBoardProtocol::getGame() {
+ChessGame * XBoardProtocol::getGame(bool allowFailure) {
   ChessGame *refgame;
   refgame=global.getGame(XBOARD_GAME);
-  if (!refgame) {
+  if (!allowFailure && refgame==NULL) {
     cerr << _("* game not found: ") << XBOARD_GAME << endl;
     exit(5);
   }
@@ -434,11 +478,12 @@ int XBoardProtocol::loadEngine() {
   global.debug("XBoardProtocol","loadEngine",EngineCommandLine);
 
   if (strlen(EngineRunDir))
-    sprintf(FullCommand,"cd %s ; %s",EngineRunDir,EngineCommandLine);
+    snprintf(FullCommand,1024,"cd %s ; %s",EngineRunDir,EngineCommandLine);
   else
     strcpy(FullCommand,EngineCommandLine);
 
   link=new PipeConnection("/bin/sh","-c",FullCommand,0,0);
+  link->setMaxWaitTime(3000.0);
 
   // only gnu chess 4 has trouble with it
   if (need_handshake)
@@ -458,8 +503,14 @@ int XBoardProtocol::loadEngine() {
       dumpGame();
     }
   }
-
+  
   global.status->setText(_("Failed to run engine."),30);
+  
+  global.output->append(_("Failed to run engine."),
+			global.Colors.TextBright,IM_NORMAL);
+  global.output->append(_("Failed command line:"),
+			global.Colors.TextBright,IM_NORMAL);
+  global.output->append(FullCommand, global.Colors.TextBright,IM_NORMAL);
   delete link;
   return 0;
 }
@@ -477,12 +528,12 @@ void XBoardProtocol::initEngine() {
 
   global.network->writeLine("new");
 
-  timecontrol.toXBoard(z);
+  timecontrol.toXBoard(z,256);
   if (z[0])
     global.network->writeLine(z);
 
   if (MaxDepth > 0) {
-    sprintf(z,"sd %d",MaxDepth);
+    snprintf(z,256,"sd %d",MaxDepth);
     global.network->writeLine(z);
   }
 
@@ -492,7 +543,8 @@ void XBoardProtocol::initEngine() {
 
 void XBoardProtocol::endInit() {
   char z[256];
-  sprintf(z,_("%s engine started."), ComputerName);
+  InitDone = true;
+  snprintf(z,256,_("%s engine started."), ComputerName);
   global.status->setText(z,15);
 }
 
@@ -533,7 +585,7 @@ void XBoardProtocol::setupPosition() {
 
     s="setboard ";
     s+=initpos.getFEN();
-    strcpy(z,s.c_str());
+    g_strlcpy(z,s.c_str(),256);
     global.network->writeLine(z);    
     just_sent_fen = true;
 
@@ -556,7 +608,7 @@ void XBoardProtocol::setupPosition() {
       for(j=0;j<8;j++) {
 	p=initpos.getPiece(i,j);
 	if ((p&COLOR_MASK) == WHITE) {
-	  sprintf(z,"%c%c%d",assoc[p&PIECE_MASK],'a'+i,1+j);
+	  snprintf(z,256,"%c%c%d",assoc[p&PIECE_MASK],'a'+i,1+j);
 	  global.network->writeLine(z);
 	}
       }
@@ -567,7 +619,7 @@ void XBoardProtocol::setupPosition() {
       for(j=0;j<8;j++) {
 	p=initpos.getPiece(i,j);
 	if ((p&COLOR_MASK) == BLACK) {
-	  sprintf(z,"%c%c%d",assoc[p&PIECE_MASK],'a'+i,1+j);
+	  snprintf(z,256,"%c%c%d",assoc[p&PIECE_MASK],'a'+i,1+j);
 	  global.network->writeLine(z);
 	}
       }
@@ -589,12 +641,13 @@ void XBoardProtocol::createGame() {
   cg=new ChessGame();
   cg->GameNumber=XBOARD_GAME;
   cg->source = GS_Engine;
-  strcpy(cg->PlayerName[EngineWhite?0:1],getComputerName());
-  strcpy(cg->PlayerName[EngineWhite?1:0],global.env.User.c_str());
+  cg->timecontrol = timecontrol;
+  g_strlcpy(cg->PlayerName[EngineWhite?0:1],getComputerName(),64);
+  g_strlcpy(cg->PlayerName[EngineWhite?1:0],global.env.User.c_str(),64);
 
   cg->MyColor=EngineWhite?BLACK:WHITE;
 
-  timecontrol.toString(cg->info0);
+  timecontrol.toString(cg->info0,64);
 
   cg->clock_regressive=timecontrol.isRegressive()?1:0;
   cg->Rated=0;
@@ -621,10 +674,10 @@ void XBoardProtocol::createGame() {
     MoveBlack = pos->sidehint ? 0 : 1;
   }
 
-  cg->updatePosition(*pos,1,pos->sidehint ? 0:1,
-		     timecontrol.startValue(),
-		     timecontrol.startValue(),
-		     "0. startpos");
+  cg->updatePosition2(*pos,1,pos->sidehint ? 0:1,
+		      1000*timecontrol.startValue(),
+		      1000*timecontrol.startValue(),
+		      "0. startpos");
   delete pos;
 
   global.gameStarted();
@@ -634,7 +687,7 @@ void XBoardProtocol::dumpGame() {
   ChessGame *cg;
 
   cg=global.getGame(XBOARD_GAME);
-  if (!cg) return;
+  if (cg==NULL) return;
 
   if (!cg->isOver()) cg->endGame("*",UNDEF);
   global.BoardList.front()->reset();
@@ -657,11 +710,11 @@ void XBoardProtocol::parseFeatures() {
       p=sf.getSToken(0);
       if (!strcmp(p,"myname")) {
 	ComputerName[63]=0;
-	strncpy(ComputerName,sf.getStarToken(0),64);
+	g_strlcpy(ComputerName,sf.getStarToken(0),64);
 	global.status->setText(ComputerName,10);
 	cg=global.getGame(XBOARD_GAME);
 	if (cg) {
-	  strncpy(cg->PlayerName[EngineWhite?0:1],ComputerName,64);
+	  g_strlcpy(cg->PlayerName[EngineWhite?0:1],ComputerName,64);
 	  cg->acknowledgeInfo();
 	}
       }
@@ -675,6 +728,10 @@ void XBoardProtocol::parseFeatures() {
 	n=atoi(nf.getNToken(0));
 	if (n) supports_setboard=true;
       }
+      if (!strcmp(p,"usermove")) {
+	n=atoi(nf.getNToken(0));
+	if (n) requires_usermove=true;
+      }
 
       if (!strcmp(p,"done")) {
 	n=atoi(nf.getNToken(0));
@@ -727,7 +784,7 @@ void XBoardProtocol::createDialog() {
   GtkWidget *bp_l,*bp_v, *pp_l=0, *pp_v=0;
   GtkWidget *h3=0,*l3=0,*path3,*b31,*l31,*h4=0,*l4=0,*path4;
 
-  eng_dlg=gtk_window_new(GTK_WINDOW_DIALOG);
+  eng_dlg=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(eng_dlg),getDialogName());
   gtk_window_set_position(GTK_WINDOW(eng_dlg),GTK_WIN_POS_CENTER);
   gtk_container_set_border_width(GTK_CONTAINER(eng_dlg),6);
@@ -764,7 +821,7 @@ void XBoardProtocol::createDialog() {
   tch = gtk_hbox_new(FALSE,2);
   gtk_box_pack_start(GTK_BOX(tv),tch,TRUE,TRUE,0);
 
-  timecontrol.toString(z);
+  timecontrol.toString(z,64);
   l2=gtk_label_new(z);  
   gtk_box_pack_start(GTK_BOX(tch),l2,TRUE,TRUE,0);
   
@@ -795,9 +852,9 @@ void XBoardProtocol::createDialog() {
   gtset(GTK_TOGGLE_BUTTON(bookmark), 1);
 
   Gtk::show(bookmark,think,te1,tl2,tl1,tabl,tv,f2,tch,l2,tce,rd[1],rd[0],
-	    v2,cfr,0);
+	    v2,cfr,NULL);
   gtk_notebook_append_page(GTK_NOTEBOOK(eng_book),bp_v,bp_l);
-  Gtk::show(bp_v,bp_l,0);
+  Gtk::show(bp_v,bp_l,NULL);
 
   // path pane
   if (want_path_pane) {
@@ -828,9 +885,9 @@ void XBoardProtocol::createDialog() {
     gtk_box_pack_start(GTK_BOX(b31),l31,FALSE,FALSE,2);
     gtk_box_pack_start(GTK_BOX(pp_v),b31,FALSE,FALSE,2);
 
-    Gtk::show(h3,l3,l31,b31,path3,h4,l4,path4,0);
+    Gtk::show(h3,l3,l31,b31,path3,h4,l4,path4,NULL);
     gtk_notebook_append_page(GTK_NOTEBOOK(eng_book),pp_v,pp_l);
-    Gtk::show(pp_v,pp_l,0);
+    Gtk::show(pp_v,pp_l,NULL);
   }
 
   // bottom box
@@ -849,7 +906,7 @@ void XBoardProtocol::createDialog() {
   gtk_box_pack_start(GTK_BOX(bhb),cancel,TRUE,TRUE,0);
   gtk_widget_grab_default(ok);
 
-  Gtk::show(ok,cancel,bhb,hs,v,0);
+  Gtk::show(ok,cancel,bhb,hs,v,NULL);
 
   gtk_signal_connect(GTK_OBJECT(ok), "clicked",
                      GTK_SIGNAL_FUNC(xboard_eng_ok),this);
@@ -884,7 +941,7 @@ void XBoardProtocol::createDialog() {
 
 void XBoardProtocol::update() {
   char z[64];
-  timecontrol.toString(z);
+  timecontrol.toString(z,64);
   gtk_label_set_text(GTK_LABEL(engctl_lbl), z);
   gtk_widget_queue_resize(engctl_lbl);
 }
@@ -895,8 +952,8 @@ void XBoardProtocol::readDialog() {
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(engctl_ewhite)))
     EngineWhite=1;
   MaxDepth=atoi(gtk_entry_get_text(GTK_ENTRY(engctl_ply)));
-  strncpy(EngineCommandLine,gtk_entry_get_text(GTK_ENTRY(engctl_engcmd)),512);
-  strncpy(EngineRunDir,gtk_entry_get_text(GTK_ENTRY(engctl_engdir)),256);
+  g_strlcpy(EngineCommandLine,gtk_entry_get_text(GTK_ENTRY(engctl_engcmd)),512);
+  g_strlcpy(EngineRunDir,gtk_entry_get_text(GTK_ENTRY(engctl_engdir)),256);
   ThinkAlways=(bool)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(think));
 
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(bookmark))) {
@@ -938,14 +995,17 @@ void XBoardProtocol::makeBookmarkCaption() {
       x=*p;
   }
   
-  ebm->timecontrol.toString(z);
-  sprintf(caption,_("Play %s as %s vs. %s (%s, maxdepth %d, think always: %s)"),
-	  ChessGame::variantName(ebm->mode),
-	  ebm->humanwhite?_("White"):_("Black"),
-	  x.c_str(),
-	  z,
-	  ebm->maxply,
-	  ebm->think?_("yes"):_("no"));
+  ebm->timecontrol.toString(z,128);
+  snprintf(caption,1024,
+	   _("Play %s as %s vs. %s (%s, maxdepth %d, think always: %s)"),
+	   ChessGame::variantName(ebm->mode),
+	   ebm->humanwhite?_("White"):
+	   _("Black"),
+	   x.c_str(),
+	   z,
+	   ebm->maxply,
+	   ebm->think?_("yes"):
+	              _("no"));
   ebm->caption = caption;
 }
 
@@ -955,8 +1015,8 @@ void XBoardProtocol::loadBookmark(EngineBookmark *bm) {
   ThinkAlways = (bm->think!=0);
   EngineWhite = 1-bm->humanwhite;
   Variant     = bm->mode;
-  strcpy(EngineRunDir, bm->directory.c_str() );
-  strcpy(EngineCommandLine, bm->cmdline.c_str() );
+  g_strlcpy(EngineRunDir, bm->directory.c_str(), 256 );
+  g_strlcpy(EngineCommandLine, bm->cmdline.c_str(), 512 );
 
   if (got_init_pos)
     EngineWhite=initpos.sidehint ? 1 : 0;
@@ -1020,10 +1080,10 @@ char * CraftyProtocol::getDialogName() {
 void CraftyProtocol::readDialog() {
   XBoardProtocol::readDialog();
   resolvePaths();
-  sprintf(EngineCommandLine,"crafty bookpath=%s logpath=%s tbpath=%s",
-	  BookPath,LogPath,LogPath);
+  snprintf(EngineCommandLine,512,"crafty bookpath=%s logpath=%s tbpath=%s",
+	   BookPath,LogPath,LogPath);
   if (!global.env.Home.empty())
-    sprintf(EngineRunDir,"%s/.eboard/craftylog",global.env.Home.c_str());
+    snprintf(EngineRunDir,512,"%s/.eboard/craftylog",global.env.Home.c_str());
   else
     strcpy(EngineRunDir,"/tmp");
 
@@ -1047,7 +1107,7 @@ void CraftyProtocol::resolvePaths() {
 
   // book path
   if (!global.env.Home.empty())
-    sprintf(z,"%s/.eboard",global.env.Home.c_str());
+    snprintf(z,256,"%s/.eboard",global.env.Home.c_str());
   else
     strcpy(z,"/tmp");
 
@@ -1059,7 +1119,7 @@ void CraftyProtocol::resolvePaths() {
   ff.addDirectory("/usr/share/crafty");
 
   if (ff.find("book.bin",zz)) {
-    p=rindex(zz,'/');
+    p=strrchr(zz,'/');
     *p=0;
   } else
     strcpy(zz,z);
@@ -1068,7 +1128,7 @@ void CraftyProtocol::resolvePaths() {
 
   // log path
   if (!global.env.Home.empty())
-    sprintf(z,"%s/.eboard/craftylog",global.env.Home.c_str());
+    snprintf(z,256,"%s/.eboard/craftylog",global.env.Home.c_str());
   else
     strcpy(z,"/tmp");
   strcpy(LogPath,z);
@@ -1104,12 +1164,12 @@ void SjengProtocol::initEngine() {
   default: z[31]=0; /* nothing! */ break;
   }
 
-  timecontrol.toXBoard(z);
+  timecontrol.toXBoard(z,32);
   if (z[0])
     global.network->writeLine(z);
 
   if (MaxDepth>0) {
-    sprintf(z,"sd %d",MaxDepth);
+    snprintf(z,32,"sd %d",MaxDepth);
     global.network->writeLine(z);
   }
 
@@ -1146,7 +1206,7 @@ void SjengProtocol::createDialog() {
     gshow(rd[i]);
     varbutton[i]=rd[i];
   }
-  Gtk::show(v,fr,0);
+  Gtk::show(v,fr,NULL);
   vl=gtk_label_new(_("Variant"));
   gshow(vl);
   gtk_notebook_append_page(GTK_NOTEBOOK(eng_book),fr,vl);
@@ -1175,13 +1235,13 @@ char * SjengProtocol::getDialogName() {
 
 void SjengProtocol::sendDrop(piece p,int x,int y) {
   ChessGame *refgame;
-  char mymove[12],ppfmm[18]; // [p]retty [p]rint [f]or [m]y [m]ove
+  char mymove[12],ppfmm[32]; // [p]retty [p]rint [f]or [m]y [m]ove
   piece clp;
 
   refgame=getGame();
   LegalityBackup=CurrentPosition;
 
-  sprintf(mymove,"P@%c%c",'a'+x,'1'+y);
+  snprintf(mymove,12,"P@%c%c",'a'+x,'1'+y);
   clp=p&PIECE_MASK;
   switch(clp) {
   case PAWN:    mymove[0]='P'; break;
@@ -1194,15 +1254,15 @@ void SjengProtocol::sendDrop(piece p,int x,int y) {
   CurrentPosition.moveDrop(p|(refgame->MyColor),x,y);
 
   if (EngineWhite)
-    sprintf(ppfmm,"%d. ... %s",MoveNumber,mymove);
+    snprintf(ppfmm,32,"%d. ... %s",MoveNumber,mymove);
   else
-    sprintf(ppfmm,"%d. %s",MoveNumber,mymove);
+    snprintf(ppfmm,32,"%d. %s",MoveNumber,mymove);
 
-  refgame->updatePosition(CurrentPosition, 
-                          MoveNumber,!MoveBlack,
-			  CLOCK_UNCHANGED,
-			  CLOCK_UNCHANGED,
-			  ppfmm);
+  refgame->updatePosition2(CurrentPosition, 
+			   MoveNumber,!MoveBlack,
+			   CLOCK_UNCHANGED,
+			   CLOCK_UNCHANGED,
+			   ppfmm);
 
   global.network->writeLine(mymove);
   advanceMove();
@@ -1228,12 +1288,12 @@ void GnuChess4Protocol::initEngine() {
 
   global.network->writeLine("new");
 
-  timecontrol.toXBoard(z,TCF_GNUCHESS4QUIRK);
+  timecontrol.toXBoard(z,32,TCF_GNUCHESS4QUIRK);
   if (z[0])
     global.network->writeLine(z);
 
   if (MaxDepth > 0) {
-    sprintf(z,"depth %d",MaxDepth);
+    snprintf(z,32,"depth %d",MaxDepth);
     global.network->writeLine(z);
   }
 
