@@ -1,11 +1,10 @@
-/* $Id: proto_xboard.h,v 1.15 2003/07/02 23:50:24 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -102,7 +101,7 @@ class XBoardProtocol : public EngineProtocol,
   void backMove();
   void advanceMove();
   void gameOver(ExtPatternMatcher &pm,GameResult gr,int hasreason=1);
-  ChessGame *getGame();
+  ChessGame *getGame(bool allowFailure=false);
 
   virtual void setupPosition();
 
@@ -120,6 +119,7 @@ class XBoardProtocol : public EngineProtocol,
   char        ComputerName[64];  
 
   bool supports_setboard;
+  bool requires_usermove;
 
   int  want_path_pane;
   bool need_handshake;
@@ -152,7 +152,9 @@ class XBoardProtocol : public EngineProtocol,
 
   char FullCommand[1024];
   GtkWidget *engctl_lbl, *engctl_ewhite, *engctl_ply,*think,*bookmark;
-  int GotResult;
+  int  GotResult;
+  bool Finalized;
+  bool InitDone;
 
   friend void xboard_eng_ok(GtkWidget *w,gpointer data);
   friend void xboard_eng_cancel(GtkWidget *w,gpointer data);
@@ -161,6 +163,12 @@ class XBoardProtocol : public EngineProtocol,
 
 };
 
+void xboard_eng_ok(GtkWidget *w,gpointer data);
+void xboard_eng_cancel(GtkWidget *w,gpointer data);
+void xboard_edit_time(GtkWidget *w,gpointer data);
+gboolean xboard_eng_delete(GtkWidget *w,GdkEvent *e,gpointer data);
+
+
 class CraftyProtocol : public XBoardProtocol {
  public:
   CraftyProtocol();
