@@ -1,11 +1,10 @@
-/* $Id: protocol.cc,v 1.28 2007/06/09 11:35:06 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +23,6 @@
 */
 
 
-
 #include "protocol.h"
 #include "global.h"
 
