@@ -1,11 +1,10 @@
-/* $Id: quickbar.cc,v 1.6 2002/07/06 21:05:11 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,23 +22,24 @@
 
 */
 
-#include <iostream.h>
+#include <iostream>
 #include <string.h>
 #include "quickbar.h"
 #include "global.h"
 #include "stl.h"
 #include "tstring.h"
 #include "script.h"
+#include "eboard.h"
 
-#include "qb1.xpm"
-#include "qb2.xpm"
-#include "qb3.xpm"
-#include "qb4.xpm"
-#include "qb5.xpm"
-#include "qb6.xpm"
-#include "qbr.xpm"
-#include "qbl.xpm"
-#include "hammer.xpm"
+#include "xpm/qb1.xpm"
+#include "xpm/qb2.xpm"
+#include "xpm/qb3.xpm"
+#include "xpm/qb4.xpm"
+#include "xpm/qb5.xpm"
+#include "xpm/qb6.xpm"
+#include "xpm/qbr.xpm"
+#include "xpm/qbl.xpm"
+#include "xpm/hammer.xpm"
 
 void qbsetup(GtkWidget *w, gpointer data) {
   (new QuickBarSetupDialog())->show();
@@ -63,15 +63,13 @@ QuickBar::QuickBar(GtkWidget *parent) {
   d0 = gdk_pixmap_create_from_xpm_d (parent->window, &m0,
 				    &style->bg[GTK_STATE_NORMAL],
 				    (gchar **) hammer_xpm);
-  config = CommandButton(d0,m0,"Setup Buttons");
-  hide = gtk_button_new_with_label("Hide!");
+  config = CommandButton(d0,m0,_("Setup Buttons"));
+  hide = gtk_button_new_with_label(_("Hide!"));
 
   gtk_box_pack_start(GTK_BOX(widget), config, FALSE,FALSE,0);
   gtk_box_pack_start(GTK_BOX(widget), hide, FALSE,FALSE,0);
 
-  gtk_widget_show(config);
-  gtk_widget_show(hide);
-
+  Gtk::show(config,hide,NULL);
   QuickBarIcons::loadIcons(parent);
 
   gtk_signal_connect(GTK_OBJECT(config), "clicked",
@@ -89,7 +87,7 @@ void QuickBar::update() {
 }
 
 void QuickBar::clear() {
-  int i;
+  unsigned int i;
   for(i=0;i<buttons.size();i++)
     gtk_container_remove(GTK_CONTAINER(widget), buttons[i]);
   // I tried a gtk_widget_destroy here but it seems gtk_container_remove
@@ -107,15 +105,15 @@ void qb_shortcut(GtkWidget *w, gpointer data) {
   v=(int *) data;
 
   if (global.QuickbarButtons[*v] != 0) {
-    strncpy(z, global.QuickbarButtons[*v]->command.c_str(), 255);
-    z[255]=0;
+    g_strlcpy(z, global.QuickbarButtons[*v]->command.c_str(), 256);
 
+    if (global.protocol == 0) return;
     if (global.network == 0) return;
     if (!global.network->isConnected()) return;
 
     if (strstr(z,"script.") == z) {
       strcpy(y,z+7);
-      sprintf(a,"> [script run from shortcut] %s",y);
+      snprintf(a,2048,_("> [script run from shortcut] %s"),y);
       global.output->append(a, global.SelfInputColor);   
       new ScriptInstance(y);
       return;
@@ -123,10 +121,10 @@ void qb_shortcut(GtkWidget *w, gpointer data) {
 
     t.set(z);
     while((p=t.token(";"))!=0) {
-      strcpy(y,p->c_str());
-      global.network->writeLine(y);
-      strcpy(y,"> [issued from shortcut] ");
-      strcat(y,p->c_str());
+      g_strlcpy(y,p->c_str(),2048);
+      global.protocol->sendUserInput(y);
+      g_strlcpy(y,_("> [issued from shortcut] "),2048);
+      g_strlcat(y,p->c_str(),2048);
       global.output->append(y, global.SelfInputColor);   
     }    
    
@@ -134,17 +132,17 @@ void qb_shortcut(GtkWidget *w, gpointer data) {
 }
 
 void QuickBar::build() {
-  int i,j;
+  unsigned int i;
+  int j;
   GtkWidget *b;
   char caption[64];
   for(i=0;i<global.QuickbarButtons.size();i++) {
     j=global.QuickbarButtons[i]->icon;
-    strncpy(caption, global.QuickbarButtons[i]->caption.c_str(), 63);
-    caption[63]=0;
+    g_strlcpy(caption, global.QuickbarButtons[i]->caption.c_str(), 64);
     b=CommandButton(arrow[j],mask[j],caption);
     gtk_box_pack_start(GTK_BOX(widget), b, FALSE, FALSE, 0);
     buttons.push_back(b);
-    gtk_widget_show(b);
+    gshow(b);
     gtk_signal_connect(GTK_OBJECT(b), "clicked",
 		       GTK_SIGNAL_FUNC(qb_shortcut), (gpointer) (& numbers[i%10]));
   }
@@ -156,7 +154,7 @@ void qbsetup_ok(GtkWidget *w, gpointer data) {
   QuickBarSetupDialog *me;
   int i;
   QButton *b;
-  char *x, *y;
+  const char *x, *y;
 
   me=(QuickBarSetupDialog *) data;
   global.dropQuickbarButtons();
@@ -205,10 +203,10 @@ void qbsetup_right(GtkWidget *w, gpointer data) {
   gtk_widget_queue_resize(me->bpix[i]);
 }
 
-QuickBarSetupDialog::QuickBarSetupDialog() : ModalDialog("Shortcut Button Setup") {
+QuickBarSetupDialog::QuickBarSetupDialog() : ModalDialog(N_("Shortcut Button Setup")) {
   GtkWidget *v, *t, *hs, *hb, *ok, *cancel, *l1, *l2, *l3, *mc;
   GtkWidget *bh[10];
-  int i;
+  unsigned int i;
 
   QuickBarIcons::loadIcons(widget);
 
@@ -217,9 +215,9 @@ QuickBarSetupDialog::QuickBarSetupDialog() : ModalDialog("Shortcut Button Setup"
 
   t=gtk_table_new(11,14,TRUE);
 
-  l3=gtk_label_new("Button Icon");
-  l1=gtk_label_new("Button Caption");
-  l2=gtk_label_new("Command");
+  l3=gtk_label_new(_("Button Icon"));
+  l1=gtk_label_new(_("Button Caption"));
+  l2=gtk_label_new(_("Command"));
   gtk_table_attach_defaults(GTK_TABLE(t), l3, 0, 2, 0, 1);
   gtk_table_attach_defaults(GTK_TABLE(t), l1, 2, 6, 0, 1);
   gtk_table_attach_defaults(GTK_TABLE(t), l2, 6, 14, 0, 1);
@@ -242,13 +240,7 @@ QuickBarSetupDialog::QuickBarSetupDialog() : ModalDialog("Shortcut Button Setup"
     gtk_table_attach_defaults(GTK_TABLE(t), bcap[i], 2, 6, i+1, i+2);
     gtk_table_attach_defaults(GTK_TABLE(t), bcom[i], 6, 14, i+1, i+2);
 
-    gtk_widget_show(bcap[i]);
-    gtk_widget_show(bcom[i]);
-
-    gtk_widget_show(br[i]);
-    gtk_widget_show(bl[i]);
-    gtk_widget_show(bpix[i]);
-    gtk_widget_show(bh[i]);
+    Gtk::show(bcap[i],bcom[i],br[i],bl[i],bpix[i],bh[i],NULL);
 
     gtk_signal_connect(GTK_OBJECT(bl[i]), "clicked",
 		       GTK_SIGNAL_FUNC(qbsetup_left), (gpointer) this);
@@ -257,9 +249,9 @@ QuickBarSetupDialog::QuickBarSetupDialog() : ModalDialog("Shortcut Button Setup"
   }
   gtk_box_pack_start(GTK_BOX(v),t,TRUE,TRUE,2);  
 
-  mc=gtk_label_new("To run multiple commands with one shortcut, separate "\
+  mc=gtk_label_new(_("To run multiple commands with one shortcut, separate "\
 		   "the commands with ; (semicolon).\n"\
-		   "To run a script from a shortcut, set command to script.ScriptName, e.g.: script.myscript.pl .");
+		   "To run a script from a shortcut, set command to script.ScriptName, e.g.: script.myscript.pl ."));
   gtk_box_pack_start(GTK_BOX(v),mc,TRUE,TRUE,2);
 
   hs=gtk_hseparator_new();
@@ -270,25 +262,15 @@ QuickBarSetupDialog::QuickBarSetupDialog() : ModalDialog("Shortcut Button Setup"
   gtk_button_box_set_spacing(GTK_BUTTON_BOX(hb), 5);
   gtk_box_pack_start(GTK_BOX(v), hb, FALSE, TRUE, 2); 
 
-  ok=gtk_button_new_with_label("Ok");
+  ok=gtk_button_new_with_label(_("Ok"));
   GTK_WIDGET_SET_FLAGS(ok,GTK_CAN_DEFAULT);
-  cancel=gtk_button_new_with_label("Cancel");
+  cancel=gtk_button_new_with_label(_("Cancel"));
   GTK_WIDGET_SET_FLAGS(cancel,GTK_CAN_DEFAULT);
   gtk_box_pack_start(GTK_BOX(hb),ok,TRUE,TRUE,0);
   gtk_box_pack_start(GTK_BOX(hb),cancel,TRUE,TRUE,0);
   gtk_widget_grab_default(ok);
 
-  gtk_widget_show(ok);
-  gtk_widget_show(cancel);
-  gtk_widget_show(hb);
-  gtk_widget_show(hs);
-  gtk_widget_show(mc);
-  gtk_widget_show(l1);
-  gtk_widget_show(l2);
-  gtk_widget_show(l3);
-  gtk_widget_show(t);
-  gtk_widget_show(v);
-
+  Gtk::show(ok,cancel,hb,hs,mc,l1,l2,l3,t,v,NULL);
   setDismiss(GTK_OBJECT(cancel),"clicked");
 
   for(i=0;i<10;i++)
@@ -321,7 +303,7 @@ QButton::QButton() {
   icon=0;
 }
 
-QButton::QButton(char *a, char *b, int c) {
+QButton::QButton(const char *a, const char *b, int c) {
   caption=a;
   command=b;
   icon=c;
@@ -381,7 +363,7 @@ void QuickBarIcons::loadIcons(GtkWidget *parent) {
 
 GtkWidget * QuickBarIcons::CommandButton(GdkPixmap *pix, GdkBitmap *mask, 
 					 char *caption) {
-  GtkWidget *b, *h, *p, *t;
+  GtkWidget *b, *h, *p, *t=0;
 
   b=gtk_button_new();
   h=gtk_hbox_new(FALSE,2);
@@ -395,8 +377,8 @@ GtkWidget * QuickBarIcons::CommandButton(GdkPixmap *pix, GdkBitmap *mask,
 
   if (caption) gtk_box_pack_start(GTK_BOX(h),t,FALSE,TRUE,2);
 
-  gtk_widget_show(p);
-  if (caption) gtk_widget_show(t);
-  gtk_widget_show(h);
+  gshow(p);
+  if (caption) gshow(t);
+  gshow(h);
   return b;
 }
