@@ -1,11 +1,10 @@
-/* $Id: script.cc,v 1.16 2007/01/20 15:58:43 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,10 +35,10 @@
 #include "stl.h"
 #include "eboard.h"
 
-#include "spiral0.xpm"
-#include "spiral1.xpm"
-#include "spiral2.xpm"
-#include "spiral3.xpm"
+#include "xpm/spiral0.xpm"
+#include "xpm/spiral1.xpm"
+#include "xpm/spiral2.xpm"
+#include "xpm/spiral3.xpm"
 
 ScriptList::ScriptList() : ModalDialog(N_("Script List")) {
   GtkWidget *v,*sw,*bh;
@@ -217,7 +216,7 @@ void ScriptInstance::initPixmaps() {
   pixmapsok=true;  
 }
 
-ScriptInstance::ScriptInstance(char *name) : WidgetProxy() {
+ScriptInstance::ScriptInstance(const char *name) : WidgetProxy() {
   GtkWidget *h, *l, *b;
   char z[256],fp[512];
 
