@@ -1,11 +1,10 @@
-/* $Id: script.h,v 1.5 2007/01/01 18:29:03 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -57,7 +56,7 @@ void script_unselect(GtkCList *cl, gint row, gint column, GdkEventButton *eb,
 
 class ScriptInstance : public WidgetProxy {
  public:
-  ScriptInstance(char *name);
+  ScriptInstance(const char *name);
 
  private:
   PipeConnection *child;
