@@ -1,11 +1,10 @@
-/* $Id: sound.h,v 1.11 2007/01/01 18:29:03 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -45,6 +44,20 @@ class SoundEventChangeListener {
   virtual void SoundEventChanged()=0;
 };
 
+class MultiBeep {
+ public:
+  MultiBeep(int _samplerate=44100, int _duration=100, int _pitch = 440, int _count = 1);
+  ~MultiBeep();
+
+  int SampleRate;
+  int Duration;
+  int Pitch;
+  int Count;
+
+  short int *data;
+  int samples;
+};
+
 class SoundEvent {
  public:
   SoundEvent();
@@ -55,7 +68,6 @@ class SoundEvent {
 
   void read(tstring &rcline);
 
-  void safePlay();
   void play();
   void playHere();
   void edit(SoundEventChangeListener *listener);
@@ -66,12 +78,15 @@ class SoundEvent {
   int  Duration;
   int  Pitch;
   int  Count;
-  char Device[64];
   char ExtraData[256];
   bool enabled;
 
  private:
-  void sine_beep(char *device,int pitch,int duration);
+  void sine_beep();
+
+  void gstPlay(const string &_input);
+  void gstBeep();
+
   char pvt[128];
 };
 
@@ -83,7 +98,7 @@ class SoundEventDialog : public ModalDialog {
  private:
   SoundEvent *obj;
   SoundEventChangeListener *hearer;
-  GtkWidget *en[5], *rd[4], *fdlg, *tme;
+  GtkWidget *en[4], *rd[4], *fdlg, *tme;
   vector<GtkWidget *> sthemes;
 
   void apply(SoundEvent *dest);
@@ -99,22 +114,5 @@ void snddlg_test(GtkWidget *w,gpointer data);
 void snddlg_browse(GtkWidget *w,gpointer data);
 void snddlg_picktheme(GtkMenuItem *w,gpointer data);
 
-class SoundSlave {
- public:
-  SoundSlave();
-  ~SoundSlave();
-  void play(SoundEvent &se);
-  void waitForEvents();
-
- private:
-  int sout[2];
-  int pid;
-
-  bool alive();
-  bool kicking();
-  void kill();
-  void run();
-};
-
 #endif
 
