@@ -1,11 +1,10 @@
-/* $Id: status.h,v 1.2 2001/04/22 21:04:55 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,9 +32,16 @@
 class Status : public WidgetProxy {
  public:
   Status();
-  void setText(char *msg);
+  void setText(const char *msg);
+  void setText(const char *msg, int secs);
   int WaitUpdate;
  private:
+  void waitUpdate();
+  void killExp();
+
+  friend gboolean st_expire(gpointer data);
+
+  int toid;
   GtkWidget *label;
 };
 
