@@ -1,11 +1,10 @@
-/* $Id: util.h,v 1.15 2003/07/05 02:48:19 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,7 +27,6 @@
 #ifndef EBOARD_UTIL_H
 #define EBOARD_UTIL_H 1
 
-#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -38,12 +36,12 @@ class FileFinder {
  public:
   FileFinder();
   ~FileFinder();
-  void addDirectory(char *dir);
+  void addDirectory(const char *dir);
   void addDirectory(string &dir);
-  void addMyDirectory(char *dir);
+  void addMyDirectory(const char *dir);
   void addMyDirectory(string &dir);
 
-  int  find(char *name,char *fullpath);
+  int  find(const char *name,char *fullpath);
   int  find(string &name, string &out);
 
   int getPathCount();
@@ -63,6 +61,7 @@ class EboardFileFinder : public FileFinder {
 class Pattern {
  public:
   Pattern();
+  virtual ~Pattern();
 
   // results: 0=match -1 = no match 1=multiple matches; on
   // matches, last contains the first character after the match
@@ -75,7 +74,7 @@ class Pattern {
 
 class ExactStringPat : public Pattern {
  public:
-  ExactStringPat(char *pat);
+  ExactStringPat(const char *pat);
   virtual ~ExactStringPat() {}
 
   virtual int tryMatch(list<char>::iterator & first,
@@ -87,7 +86,7 @@ class ExactStringPat : public Pattern {
 // same thing, case insensitive
 class CIExactStringPat : public ExactStringPat {
  public:
-  CIExactStringPat(char *pat) : ExactStringPat(pat) { }
+  CIExactStringPat(const char *pat) : ExactStringPat(pat) { }
   virtual ~CIExactStringPat() {}
   
   int tryMatch(list<char>::iterator & first,
@@ -112,7 +111,7 @@ class SetPat : public Pattern {
   SetPat();
 
   void addToSet(char c);
-  void addToSet(char *s);
+  void addToSet(const char *s);
   void clearSet();
 
   void includeUppercase();
@@ -158,7 +157,7 @@ class PercSSetPat : public SetPat {
   static bool theset[256];
 };
 
-// %S - A-Z,a-z,0-9
+// %S - A-Z,a-z,0-9,_
 class PercUpperSSetPat : public SetPat {
  public:
   PercUpperSSetPat();
@@ -213,7 +212,7 @@ class PatternMatcher {
 
   void append(Pattern *p);
   void reset();
-  int  match(char *stryng); // binds local 'data' list
+  int  match(const char *stryng); // binds local 'data' list
   int  match();             // uses PatternBinder bound data
 
   char *getToken(int index);
@@ -243,7 +242,7 @@ class ExtPatternMatcher : public PatternMatcher {
  public:
   ExtPatternMatcher();
 
-  void set(char *hrp); // set from a human readable pattern
+  void set(const char *hrp); // set from a human readable pattern
 
   // for patterns created with "set", these are the %s , %n
   char *getSToken(int index);
@@ -318,4 +317,18 @@ class zifstream {
 
 }; 
 
+class Timestamp {
+ public:
+  Timestamp(int sec, int usec);
+  Timestamp();
+
+  Timestamp & operator=(Timestamp &t);
+  double operator-(Timestamp &t);
+  static Timestamp & now();
+
+ private:
+  int S,U;
+
+};
+
 #endif
