@@ -1,11 +1,10 @@
-/* $Id: widgetproxy.h,v 1.28 2007/02/05 19:37:56 bergo Exp $ */
-
 /*
 
     eboard - chess client
-    http://eboard.sourceforge.net
-    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
-    bergo@seul.org
+    http://www.bergo.eng.br/eboard
+    https://github.com/fbergo/eboard
+    Copyright (C) 2000-2016 Felipe Bergo
+    fbergo/at/gmail/dot/com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -40,10 +39,10 @@ class WidgetProxy {
   virtual void hide();
   virtual void repaint();
 
-  void setIcon(char **xpmdata,const char *text);
+  void setIcon(const char **xpmdata,const char *text);
   void restorePosition(WindowGeometry *wg);
 
-  GtkWidget * PixButton(char **xpmicon, char *caption);
+  GtkWidget * PixButton(const char **xpmicon, const char *caption);
 
   GtkWidget * scrollBox(GtkWidget *child);
 
@@ -60,11 +59,11 @@ class BoxedLabel : public WidgetProxy {
 
 class ModalDialog : public WidgetProxy {
  public:
-  ModalDialog(char *title);
+  ModalDialog(const char *title);
   virtual ~ModalDialog();
   virtual void show();
 
-  void setDismiss(GtkObject *obj,char *sig);
+  void setDismiss(GtkObject *obj,const char *sig);
   void setCloseable(bool v);
 
   void release();
@@ -86,7 +85,7 @@ gint modal_closereq(GtkWidget * widget,
 
 class NonModalDialog : public ModalDialog {
  public:
-  NonModalDialog(char *title);
+  NonModalDialog(const char *title);
   ~NonModalDialog();
   virtual void show();
 
@@ -195,7 +194,7 @@ class LayoutBox {
   int  substringWidth(int f,const char *s,int len);
   int  fontHeight(int f);
 
-  bool drawBoxedString(int x,int y,int w,int h,int f,char *s,char *sep);
+  bool drawBoxedString(int x,int y,int w,int h,int f,const char *s,const char *sep);
   bool prepared();
 
   void consumeTop(int v);
